#import "ImageShowController.h"
#import <QuartzCore/QuartzCore.h>

@interface ImageShowController()
@property (retain) NSImageView *currentImageView;
@end

@implementation ImageShowController

@synthesize currentImageView;
@synthesize imageShowView;
@synthesize images;

@dynamic indexOfCurrentImage;
- (NSUInteger)indexOfCurrentImage
{
    return indexOfCurrentImage;
}

- (void)setIndexOfCurrentImage:(NSUInteger)newIndex
{   
    indexOfCurrentImage = newIndex;
    
    NSImageView *newImageView = [[[NSImageView alloc] 
                    initWithFrame:[[self imageShowView] bounds]] 
                                            autorelease];
    
    [newImageView setImageFrameStyle:NSImageFrameGrayBezel];
    [newImageView setAutoresizingMask:NSViewMinXMargin | 
                                    NSViewWidthSizable | 
                                      NSViewMaxXMargin | 
                                      NSViewMinYMargin | 
                                      NSViewHeightSizable | 
                                      NSViewMaxYMargin];
    
    [newImageView setImageScaling:
            NSImageScaleProportionallyUpOrDown];
    
    NSImage *image = [[self images] objectAtIndex:
                                [self indexOfCurrentImage]];
    
    [newImageView setImage:image];
    
    if([self currentImageView] == nil) 
    {
        [[imageShowView animator] addSubview:newImageView];
    } else 
    {
        [[imageShowView animator] 
            replaceSubview:[self currentImageView] 
                      with:newImageView];
    }
    
    [self setCurrentImageView:newImageView];
}

- (void)awakeFromNib
{
    [self setCurrentImageView:nil];
    [imageShowView setWantsLayer:YES];
    indexOfCurrentImage = 0;
    CATransition *transition = [CATransition animation];
    [transition setDuration:1.0];
    [transition setType:kCATransitionPush];
    NSDictionary *newAnimations = [NSDictionary 
                            dictionaryWithObject:transition 
                                          forKey:@"subviews"];
    
    [[imageShowView animator] setAnimations:newAnimations];
}

- (IBAction)nextImage:(id)sender 
{
    [[imageShowView animator] 
                setValue:kCATransitionFromRight 
              forKeyPath:@"animations.subviews.subtype"];
    
    if(([self indexOfCurrentImage]) >= ([images count] - 1)) 
    {
        [self setIndexOfCurrentImage:0];
    } else 
    {
        [self setIndexOfCurrentImage:
                            [self indexOfCurrentImage] + 1];
    }
}

- (IBAction)previousImage:(id)sender 
{
    [[imageShowView animator] 
                setValue:kCATransitionFromLeft 
              forKeyPath:@"animations.subviews.subtype"];
    
    if(([self indexOfCurrentImage]) == 0) 
    {
        [self setIndexOfCurrentImage:[[self images] count] - 1];
    } else 
    {
        [self setIndexOfCurrentImage:
                    [self indexOfCurrentImage] - 1];
    }
}

@end


