//
//  TableBaizeView.m
//  Card Game
//
//  Created by Amin Negm-Awad on Mi.,23.06.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "TableBaizeView.h"
#import "CardStackView.h"

@implementation TableBaizeView


- (void)resizeSubviews 
{
   NSSize cardSize;
   NSRect rect;
   NSRect bounds = [self bounds];
   
   // Kartenausmasse bestimmen: Die Zweichenraeume sollen jeweils 
   // 10 % der Breite einnehmen. Da neun Zwischenraeume und zwei
   // seitliche existieren, entspricht das 1,1 Karten, insgesamt 
   // haben wir also 11,1 Kartenbreiten
   CGFloat border = bounds.size.width / 111; 
   cardSize.width = bounds.size.width / 11.1; 
   cardSize.height 
   = [[CardStackView class] cardHeightForWidth:cardSize.width];
   
   // Hieraus ergibt sich das Rechteck, in dem die Karten liegen 
   bounds = NSInsetRect( bounds, border, border );
   
   // Die einer Karte ist zugleich die Hoehe der unteren waage- 
   // rechten Stapel. Sie belgen jeweils die Hälte des Platzes 
   rect.origin = bounds.origin;
   rect.size.width = NSWidth( bounds ) / 2;
   rect.size.height = cardSize.height; 
   
   NSLog( @"bounds aligned: %@", NSStringFromRect( rect ) ); 
   rect = [self centerScanRect:rect]; 
   NSLog( @"device aligned: %@", NSStringFromRect( rect ) );
   [depotView setFrame:rect];
   
   rect.origin.x = NSMaxX( rect ); 
   
   rect = [self centerScanRect:rect];
   [targetView setFrame:rect];
   
   // Fuer die oberen Karten ergibt sich daraus das Rechteck 
   rect.origin.x = NSMinX( bounds ); 
   rect.origin.y = NSMaxY( rect ) + border; 
   rect.size.width = cardSize.width;
   rect.size.height = NSHeight( bounds) - NSMinY( rect );
   
   // die einzelnen Views einfach nacheinander setzen 
   for( CardStackView* playerView in playerViews ) {
      rect = [self centerScanRect:rect];
      [playerView setFrame:rect]; 
      rect.origin.x += rect.size.width + border;
   }
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
       // Zehn obere Stapel
       CardStackView* view;
       playerViews = [[NSMutableArray alloc] init];
       NSUInteger index; 
       for( index = 0; index < 10; index++ ) {   
          view = [[[CardStackView alloc] 
                                  initWithFrame:NSZeroRect]
                                  autorelease]; 
          [view setAlignment:NSMaxYEdge];
          [self addSubview:view];
          [playerViews addObject:view];
       }
       
       // Depot 
       view = [[[CardStackView alloc]
                               initWithFrame:NSZeroRect]
                               autorelease];
       [view setAlignment:NSMinXEdge];
       [self addSubview:view]; 
       depotView = view;
       
       // Ablage 
       view = [[[CardStackView alloc]
                               initWithFrame:NSZeroRect]
                               autorelease];
       [view setAlignment:NSMaxXEdge];
       [self addSubview:view]; 
       targetView = view;
       
       [self resizeSubviews];
    }
    return self;
}

- (void) dealloc
{
   [playerViews release]; 
   playerViews = nil; 
   depotView = nil; 
   targetView = nil;
   
   [super dealloc];
}

- (void)resizeSubviewsWithOldSize:(NSSize)oldSize { 
   [self resizeSubviews];
}

- (void)drawRect:(NSRect)dirtyRect {
   NSColor* lighterColor = [NSColor colorWithCalibratedRed:0.2 
                                                     green:0.7 
                                                      blue:0.2
                                                     alpha:1.0];

   NSColor* darkerColor = [NSColor colorWithCalibratedRed:0.0 
                                                    green:0.5 
                                                     blue:0.0
                                                    alpha:1.0];

   NSGradient* gradient; 
   gradient = [[[NSGradient alloc]
                            initWithStartingColor:lighterColor
                                      endingColor:darkerColor] 
                            autorelease];
   
   [gradient            	drawInRect:[self bounds] 
             relativeCenterPosition:NSMakePoint( -0.5, +0.5 )];
   
}
@end
