//
//  CardStackView.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "CardStackView.h"

#import "BezierPathCardsAddition.h"

// Kartenbemassung
static const float cardSizeRatio = 1.6;
static const float edgeRadius = 0.1;
static const NSSize stepSize = { 0.21, 0.11 };

// Positionen in Kartengroesse
static const NSPoint labelLocation = { 0.05, 0.9 };
static const CGFloat labelHeight   = 0.08;
@interface CardStackView( PrivateMethodsAddition )
- (NSRect)rectForCardWithIndex:(NSUInteger)index 
                         count:(NSUInteger)count
                   getIsJammed:(BOOL*)jammed;
@end

@implementation CardStackView
@synthesize alignment; // cards entfernt 

- (NSArray*)cards { return cards; }
- (void)setCards:(NSArray*)value
{
   // Aenderungsrechtecke herausfinden
   NSRect dirtyRect;
   
   // Die Anzahl der vorhandene Karten und Stauchung holen 
   NSUInteger oldCount = [cards count];
   NSUInteger newCount = [value count];
   
   BOOL oldJammed;
   BOOL newJammed;
   [self rectForCardWithIndex:0 
                        count:oldCount
                  getIsJammed:&oldJammed];
   [self rectForCardWithIndex:0 
                        count:newCount
                  getIsJammed:&newJammed];
   
   // Wenn mindestens ein Stapel gestaucht ist und sich die An-
   // zahl der Karten veraendert, muss ohnehin das gesamte View
   // neu gezeichnet werden
   if( (oldJammed || newJammed) && (oldCount != newCount ) ) {
      [self setNeedsDisplay:YES];
      
      // Andernfalls wissen wir hier, dass entweder keine Stauchung 
      // vorliegt, der count also für die Berechnung der Rechtecke 
      // unbeachtlich ist, oder aber beide Stapel dieselbe Anzahl
      // von Karten enthalten, so dass beide counts gleich sind.
   } else {
      // Anzahl der gemeinsamen Karten
      NSUInteger count = (oldCount < newCount)?oldCount:newCount;
      NSUInteger index;
      for( index = 0; index < count; index++ ) {
         id oldCard = [cards objectAtIndex:index];
         id newCard = [value objectAtIndex:index];
         
         // Unterschiedlicher Wert
         NSNumber* oldValue = [oldCard valueForKey:@"value"];
         NSNumber* newValue = [newCard valueForKey:@"value"];
         if( ![oldValue isEqualToNumber:newValue] ) {
            dirtyRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:&oldJammed];
            [self setNeedsDisplayInRect:dirtyRect]; 
            continue;
            //////////
         }
         
         // Unterschiedliche Farbe
         NSNumber* oldSuit = [oldCard valueForKey:@"suit"];
         NSNumber* newSuit = [newCard valueForKey:@"suit"];
         if( ![oldSuit isEqualToNumber:newSuit] ) {
            dirtyRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:&oldJammed];
            [self setNeedsDisplayInRect:dirtyRect];
            continue;
            //////////
         }
      }
      
      // Unterschiedliche Stapelhoehen beruecksichtigen:
      // index zeigt bisher auf erste "ueberstehende" Karte des 
      // groesseren Stapels
      
      // Letzte Karte holen
      count  = (oldCount > newCount)?oldCount:newCount;
      
      // Den Bereich zwischen den Karten markieren
      NSRect lastRect;
      dirtyRect = [self rectForCardWithIndex:index
                                       count:count
                                 getIsJammed:&oldJammed];
      lastRect  =  [self rectForCardWithIndex:count-1
                                        count:count
                                  getIsJammed:&oldJammed];
      
      // Der gesamte Bereich muss neu gezeichnet werden. 
      dirtyRect = NSUnionRect( dirtyRect, lastRect );
      
      [self setNeedsDisplayInRect:dirtyRect];   
   }
   
   // Standardaccessor
   if( value != cards ) {
      [cards release];
      cards = [value copy];
   }
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
       self.alignment = NSMaxYEdge;
       
       NSDictionary* card;
       NSMutableArray* initCards = [NSMutableArray array];
       card = [NSDictionary dictionaryWithObjectsAndKeys:
               [NSNumber numberWithUnsignedInteger:1], @"suit",
               [NSNumber numberWithUnsignedInteger:1], @"value",
               [NSNumber numberWithBool:YES], @"disclosed",
               nil];
       [initCards addObject:card];
       card = [NSDictionary dictionaryWithObjectsAndKeys:
               [NSNumber numberWithUnsignedInteger:2], @"suit",
               [NSNumber numberWithUnsignedInteger:7], @"value",
               [NSNumber numberWithBool:YES], @"disclosed",
               nil];
       [initCards addObject:card];
       card = [NSDictionary dictionaryWithObjectsAndKeys:
               [NSNumber numberWithUnsignedInteger:2], @"suit",
               [NSNumber numberWithUnsignedInteger:7], @"value",
               [NSNumber numberWithBool:YES], @"disclosed",
               nil];
       [initCards addObject:card];
       self.cards = initCards;
       self.alignment = NSMaxXEdge;
    }
    return self;
}

- (void) dealloc
{
   self.cards = nil;
   self.alignment = 0;
   [super dealloc];
}

- (void)drawUpperLabelWithSuit:(NSUInteger)suit
                         value:(NSUInteger)value
                        inRect:(NSRect)rect
{
   NSBezierPath* path;
   
   // Berechne das Label-Rect
   NSPoint location;
   location.x 
   = NSMinX( rect ) + NSWidth( rect )  * labelLocation.x; 
   location.y 
   = NSMinY( rect ) + NSHeight( rect ) * labelLocation.y; 
   CGFloat height = rect.size.height * labelHeight;
   
   // Farbsymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForSuit:suit];
   
   [path transformToLocation:location height:height];
   
   [path fill];
   location.x = NSMaxX( [path bounds] ) + 0.01 * NSWidth( rect );
   
   // Wertesymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForValue:value];
   
   [path transformToLocation:location height:height];
   
   [path fill];
}

- (void)drawLowerLabelWithSuit:(NSUInteger)suit
                         value:(NSUInteger)value
                        inRect:(NSRect)rect
{
   NSBezierPath* path;
   
   // Berechne das Label-Rect
   NSPoint location;
   location.x 
      = NSMaxX( rect ) - NSWidth( rect )  * labelLocation.x; 
   location.y 
      = NSMaxY( rect ) - NSHeight( rect ) * labelLocation.y; 
   CGFloat height = -1.0 * rect.size.height * labelHeight;
   
   // Farbsymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForSuit:suit];
   
   [path transformToLocation:location height:height];
   
   [path fill];
   location.x = NSMinX( [path bounds] ) - 0.01 * NSWidth( rect );
   
   // Wertesymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForValue:value];
   
   [path transformToLocation:location height:height];
   
   [path fill];
}

- (void)drawCardFaceWithSuit:(NSUInteger)suit
                       value:(NSUInteger)value
                      inRect:(NSRect)rect 
{
   if( suit < 2 ) {
      [[NSColor redColor] set];
   } else {
      [[NSColor blackColor] set];
   }
   
   [self drawUpperLabelWithSuit:suit value:value inRect:rect];
   [self drawLowerLabelWithSuit:suit value:value inRect:rect];
}

- (void)drawCardBorderInRect:(NSRect)rect
{
   NSBezierPath* path;
   
   CGFloat radius = NSWidth( rect ) * edgeRadius;
   
   path = [NSBezierPath bezierPathWithRoundedRect:rect 
                                          xRadius:radius 
                                          yRadius:radius];
   [path setLineWidth:0.0];
   [[NSColor whiteColor] setFill];
   [path fill];
   
   [[NSColor blackColor] setStroke];
   [path stroke];
}
+ (CGFloat)cardHeightForWidth:(CGFloat)width {
   return width * cardSizeRatio;
}

+ (CGFloat)cardWidthForHeight:(CGFloat)height {
   return height / cardSizeRatio;
}

- (NSRect)rectForCardWithIndex:(NSUInteger)index 
                         count:(NSUInteger)count
                   getIsJammed:(BOOL*)isJammed
{
   CGFloat xStep;
   CGFloat yStep;
   
   NSRect rect = [self bounds];
   
   // Startwert und Schrittweite nach Ausrichtungen setzen
   switch( self.alignment ) {
      case NSMinXEdge: // linksbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x = 0.0;
         xStep = NSWidth( rect ) * +stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMaxXEdge: // rechtbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x 
         = NSMaxX( [self bounds] ) - NSWidth( rect );
         xStep = NSWidth( rect ) * -stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMinYEdge: // untenbuendig (hier nicht verwendet)
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y = 0.0;
         xStep = 0.0;
         yStep = NSHeight( rect ) * +stepSize.height;
         break;
         
      case NSMaxYEdge: // obenbuendig
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y 
         = NSMaxY( [self bounds] ) - NSHeight( rect );
         xStep = 0.0;
         yStep = NSHeight( rect ) * -stepSize.height;
         
         break;
      default:
         break;
   }
   
   // Ueberlaeufe abfangen
   BOOL jammed = NO;
   CGFloat maxStep;
   
   count--;
   if( count > 0 ) {
      maxStep = (NSHeight([self bounds])-NSHeight(rect)) / count;
      if( yStep < 0.0 ) {
         if( -yStep > maxStep ) {
            yStep = -maxStep;
            jammed = YES;
         }
      } else {
         if( yStep > maxStep ) {
            yStep = maxStep;
            jammed = YES;
         }
      }
      
      maxStep = (NSWidth([self bounds])-NSWidth(rect)) / count;
      if( xStep < 0.0 ) {
         if( -xStep > maxStep ) {
            xStep = -maxStep;
            jammed = YES;
         } 
      } else {
         if( xStep > maxStep ) {
            xStep = +maxStep;
            jammed = YES;
         } 
      }         
   }
   if( isJammed != NULL ) {
      *isJammed = jammed;
   }
   
   
   // Verschiebe Karte um Index
   rect = NSOffsetRect( rect, xStep * index, yStep * index );
   if( self.alignment == NSMaxYEdge ) {
      rect.origin.x += 4.0 * index;
   }
   
   // Pixelmaessig ausrichten
   rect = [self convertRectToBase:rect]; 
   rect.origin.x = ceilf( rect.origin.x ) + 0.5;
   rect.origin.y = ceilf( rect.origin.y ) + 0.5;
   rect.size.width =floorf( rect.size.width ) - 2.0;
   rect.size.height =floorf( rect.size.height ) - 2.0;
   rect = [self convertRectFromBase:rect]; 
   
   return rect;      
}

- (NSRect)printRectForCardWithIndex:(NSUInteger)index 
                         count:(NSUInteger)count
                   getIsJammed:(BOOL*)isJammed
{
   CGFloat xStep;
   CGFloat yStep;
   
   NSRect rect = [self bounds];
   
   // Startwert und Schrittweite nach Ausrichtungen setzen
   switch( self.alignment ) {
      case NSMinXEdge: // linksbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x = 0.5;
         xStep = NSWidth( rect ) * +stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMaxXEdge: // rechtbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x 
         = NSMaxX( [self bounds] ) - NSWidth( rect ) - 0.5;
         xStep = NSWidth( rect ) * -stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMinYEdge: // untenbuendig (hier nicht verwendet)
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y = 0.5;
         xStep = 0.0;
         yStep = NSHeight( rect ) * +stepSize.height;
         break;
         
      case NSMaxYEdge: // obenbuendig
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y 
         = NSMaxY( [self bounds] ) - NSHeight( rect ) - 0.5;
         xStep = 0.0;
         yStep = NSHeight( rect ) * -stepSize.height;
         
         break;
      default:
         break;
   }
   
   // Ueberlaeufe abfangen
   BOOL jammed = NO;
   CGFloat maxStep;
   
   count--;
   if( count > 0 ) {
      maxStep = (NSHeight([self bounds])-NSHeight(rect)) / count;
      if( yStep < 0.0 ) {
         if( -yStep > maxStep ) {
            yStep = -maxStep;
            jammed = YES;
         }
      } else {
         if( yStep > maxStep ) {
            yStep = maxStep;
            jammed = YES;
         }
      }
      
      maxStep = (NSWidth([self bounds])-NSWidth(rect)) / count;
      if( xStep < 0.0 ) {
         if( -xStep > maxStep ) {
            xStep = -maxStep;
            jammed = YES;
         } 
      } else {
         if( xStep > maxStep ) {
            xStep = +maxStep;
            jammed = YES;
         } 
      }         
   }
   if( isJammed != NULL ) {
      *isJammed = jammed;
   }
   
   
   // Verschiebe Karte um Index
   rect = NSOffsetRect( rect, xStep * index, yStep * index );
   
   return rect;      
}

- (BOOL)wantsDefaultClipping { return NO; }

- (void)drawRect:(NSRect)rect {
   NSEraseRect( [self bounds] );

   NSUInteger suit;
   NSUInteger value;
   NSUInteger index;
   
   NSUInteger count = [self.cards count];
   for( index = 0; index < count; index++ ) {
	   id card = [self.cards objectAtIndex:index];
      NSRect cardRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:NULL];
      if( [self needsToDrawRect:cardRect] ) {
         [self drawCardBorderInRect:cardRect];
         suit = [[card valueForKey:@"suit"] integerValue];
         value = [[card valueForKey:@"value"] integerValue];
         [self drawCardFaceWithSuit:suit 
                              value:value 
                             inRect:cardRect];
      }
   }   
   // Aktuelles Rechteck markieren
   [[NSColor colorWithCalibratedWhite:0.0 alpha:0.2] setFill];
   [NSBezierPath fillRect:rect];
   
   // Rechteckteile markieren
   const NSRect* rects;
   NSInteger rectsCount;
   [self getRectsBeingDrawn:&rects count:&rectsCount]; 
   for( index = 0; index < rectsCount; index++ ) {
      [NSBezierPath fillRect:rects[index]];      
   }   
}

- (IBAction)createTiff:(id)sender
{
   NSSize imageSize = [self bounds].size;
   NSRect cardRect = NSInsetRect( [self bounds], 0.5, 0.5 );
   NSImage* image 
   = [[[NSImage alloc] initWithSize:imageSize] autorelease];
   [image lockFocus]; 
   NSFrameRect( cardRect );
   [self drawCardFaceWithSuit:2 value:0 inRect:cardRect];
   [image unlockFocus]; 
   NSData* tiff = [image TIFFRepresentation]; 
   NSString* path = @"~/Desktop/Bild.tiff";
   path = [path stringByExpandingTildeInPath];
   [tiff writeToFile:path atomically:YES];
}
@end
