//
//  CardStackView.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "CardStackView.h"

#import "BezierPathCardsAddition.h"

// Kartenbemassung
static const float cardSizeRatio = 1.6;
static const float edgeRadius = 0.1;
static const NSSize stepSize = { 0.21, 0.11 };

// Positionen in Kartengroesse
static const NSPoint labelLocation = { 0.05, 0.9 };
static const CGFloat labelHeight   = 0.08;
@interface CardStackView( PrivateMethodsAddition )
- (NSRect)rectForCardWithIndex:(NSUInteger)index 
                         count:(NSUInteger)count
                   getIsJammed:(BOOL*)jammed;
@end

@implementation CardStackView
@synthesize alignment, draggedCardsCount, dragRect; 

- (void)setSelectedCardsIndices:(NSIndexSet*)value 
{ 
   if( value != selectedCardsIndices ) {
      [selectedCardsIndices release];
      selectedCardsIndices = [value retain];
   }
   [self setNeedsDisplay:YES];
}

- (NSIndexSet*)selectedCardsIndices 
{
   return selectedCardsIndices;
}

- (NSArray*)cards { return cards; }
- (void)setCards:(NSArray*)value
{
   // Aenderungsrechtecke herausfinden
   NSRect dirtyRect;
   
   // Die Anzahl der vorhandene Karten und Stauchung holen 
   NSUInteger oldCount = [cards count];
   NSUInteger newCount = [value count];
   
   BOOL oldJammed;
   BOOL newJammed;
   [self rectForCardWithIndex:0 
                        count:oldCount
                  getIsJammed:&oldJammed];
   [self rectForCardWithIndex:0 
                        count:newCount
                  getIsJammed:&newJammed];
   
   // Wenn mindestens ein Stapel gestaucht ist und sich die An-
   // zahl der Karten veraendert, muss ohnehin das gesamte View
   // neu gezeichnet werden
   if( (oldJammed || newJammed) && (oldCount != newCount ) ) {
      [self setNeedsDisplay:YES];
      
      // Andernfalls wissen wir hier, dass entweder keine Stauchung 
      // vorliegt, der count also für die Berechnung der Rechtecke 
      // unbeachtlich ist, oder aber beide Stapel dieselbe Anzahl
      // von Karten enthalten, so dass beide counts gleich sind.
   } else {
      // Anzahl der gemeinsamen Karten
      NSUInteger count = (oldCount < newCount)?oldCount:newCount;
      NSUInteger index;
      for( index = 0; index < count; index++ ) {
         id oldCard = [cards objectAtIndex:index];
         id newCard = [value objectAtIndex:index];
         
         // Unterschiedlicher Wert
         NSNumber* oldValue = [oldCard valueForKey:@"value"];
         NSNumber* newValue = [newCard valueForKey:@"value"];
         if( ![oldValue isEqualToNumber:newValue] ) {
            dirtyRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:&oldJammed];
            [self setNeedsDisplayInRect:dirtyRect]; 
            continue;
            //////////
         }
         
         // Unterschiedliche Farbe
         NSNumber* oldSuit = [oldCard valueForKey:@"suit"];
         NSNumber* newSuit = [newCard valueForKey:@"suit"];
         if( ![oldSuit isEqualToNumber:newSuit] ) {
            dirtyRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:&oldJammed];
            [self setNeedsDisplayInRect:dirtyRect];
            continue;
            //////////
         }
      }
      
      // Unterschiedliche Stapelhoehen beruecksichtigen:
      // index zeigt bisher auf erste "ueberstehende" Karte des 
      // groesseren Stapels
      
      // Letzte Karte holen
      count  = (oldCount > newCount)?oldCount:newCount;
      
      // Den Bereich zwischen den Karten markieren
      NSRect lastRect;
      dirtyRect = [self rectForCardWithIndex:index
                                       count:count
                                 getIsJammed:&oldJammed];
      lastRect  =  [self rectForCardWithIndex:count-1
                                        count:count
                                  getIsJammed:&oldJammed];
      
      // Der gesamte Bereich muss neu gezeichnet werden. 
      dirtyRect = NSUnionRect( dirtyRect, lastRect );
      
      [self setNeedsDisplayInRect:dirtyRect];   
   }
   
   // Standardaccessor
   if( value != cards ) {
      [cards release];
      cards = [value copy];
   }
}


- (void)drawUpperLabelWithSuit:(NSUInteger)suit
                         value:(NSUInteger)value
                        inRect:(NSRect)rect
{
   NSBezierPath* path;
   
   // Berechne das Label-Rect
   NSPoint location;
   location.x 
   = NSMinX( rect ) + NSWidth( rect )  * labelLocation.x; 
   location.y 
   = NSMinY( rect ) + NSHeight( rect ) * labelLocation.y; 
   CGFloat height = rect.size.height * labelHeight;
   
   // Farbsymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForSuit:suit];
   
   [path transformToLocation:location height:height];
   
   [path fill];
   location.x = NSMaxX( [path bounds] ) + 0.01 * NSWidth( rect );
   
   // Wertesymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForValue:value];
   
   [path transformToLocation:location height:height];
   
   [path fill];
}

- (void)drawLowerLabelWithSuit:(NSUInteger)suit
                         value:(NSUInteger)value
                        inRect:(NSRect)rect
{
   NSBezierPath* path;
   
   // Berechne das Label-Rect
   NSPoint location;
   location.x 
      = NSMaxX( rect ) - NSWidth( rect )  * labelLocation.x; 
   location.y 
      = NSMaxY( rect ) - NSHeight( rect ) * labelLocation.y; 
   CGFloat height = -1.0 * rect.size.height * labelHeight;
   
   // Farbsymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForSuit:suit];
   
   [path transformToLocation:location height:height];
   
   [path fill];
   location.x = NSMinX( [path bounds] ) - 0.01 * NSWidth( rect );
   
   // Wertesymbol erzeugen, anpassen und ausgeben
   path = [NSBezierPath bezierPath];
   [path moveToPoint:NSMakePoint( 0.0, 0.0)];
   [path appendBezierPathForValue:value];
   
   [path transformToLocation:location height:height];
   
   [path fill];
}

- (void)drawCardFaceWithSuit:(NSUInteger)suit
                       value:(NSUInteger)value
                      inRect:(NSRect)rect 
{
   if( suit < 2 ) {
      [[NSColor redColor] set];
   } else {
      [[NSColor blackColor] set];
   }
   
   [self drawUpperLabelWithSuit:suit value:value inRect:rect];
   [self drawLowerLabelWithSuit:suit value:value inRect:rect];
}

- (void)drawCardBorderInRect:(NSRect)rect selected:(BOOL)selected
{
   NSBezierPath* path;
   
   CGFloat radius = NSWidth( rect ) * edgeRadius;
   
   path = [NSBezierPath bezierPathWithRoundedRect:rect 
                                          xRadius:radius 
                                          yRadius:radius];
   NSGraphicsContext* context 
   = [NSGraphicsContext currentContext];
   if( [context isDrawingToScreen] ) {
      [path setLineWidth:0.0];
   } else {
      [path setLineWidth:1.0];
   }
   // Selektierte Karten: 
   if( selected ) {
      [[NSColor selectedControlColor] setFill];
   } else {
      [[NSColor whiteColor] setFill];
   }
   [path fill];
   
   [[NSColor blackColor] setStroke];
   [path stroke];
}
+ (CGFloat)cardHeightForWidth:(CGFloat)width {
   return width * cardSizeRatio;
}

+ (CGFloat)cardWidthForHeight:(CGFloat)height {
   return height / cardSizeRatio;
}

- (NSRect)rectForCardWithIndex:(NSUInteger)index 
                         count:(NSUInteger)count
                   getIsJammed:(BOOL*)isJammed
{
   CGFloat xStep;
   CGFloat yStep;
   
   NSRect rect = [self bounds];
   
   // Startwert und Schrittweite nach Ausrichtungen setzen
   switch( self.alignment ) {
      case NSMinXEdge: // linksbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x = 0.0;
         xStep = NSWidth( rect ) * +stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMaxXEdge: // rechtbuendig
         rect.size.width 
         = [[self class] cardWidthForHeight:NSHeight( rect )];
         rect.origin.x 
         = NSMaxX( [self bounds] ) - NSWidth( rect );
         xStep = NSWidth( rect ) * -stepSize.width;
         yStep = 0.0;
         break;
         
      case NSMinYEdge: // untenbuendig (hier nicht verwendet)
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y = 0.0;
         xStep = 0.0;
         yStep = NSHeight( rect ) * +stepSize.height;
         break;
         
      case NSMaxYEdge: // obenbuendig
         rect.size.height
         = [[self class] cardHeightForWidth:NSWidth( rect )];
         rect.origin.y 
         = NSMaxY( [self bounds] ) - NSHeight( rect );
         xStep = 0.0;
         yStep = NSHeight( rect ) * -stepSize.height;
         
         break;
      default:
         break;
   }
   
   // Ueberlaeufe abfangen
   BOOL jammed = NO;
   CGFloat maxStep;
   
   count--;
   if( count > 0 ) {
      maxStep = (NSHeight([self bounds])-NSHeight(rect)) / count;
      if( yStep < 0.0 ) {
         if( -yStep > maxStep ) {
            yStep = -maxStep;
            jammed = YES;
         }
      } else {
         if( yStep > maxStep ) {
            yStep = maxStep;
            jammed = YES;
         }
      }
      
      maxStep = (NSWidth([self bounds])-NSWidth(rect)) / count;
      if( xStep < 0.0 ) {
         if( -xStep > maxStep ) {
            xStep = -maxStep;
            jammed = YES;
         } 
      } else {
         if( xStep > maxStep ) {
            xStep = +maxStep;
            jammed = YES;
         } 
      }         
   }
   if( isJammed != NULL ) {
      *isJammed = jammed;
   }
   
   
   // Verschiebe Karte um Index
   rect = NSOffsetRect( rect, xStep * index, yStep * index );
   
   // Pixelmaessig ausrichten
   rect = [self convertRectToBase:rect]; 
   rect.origin.x = ceilf( rect.origin.x ) + 0.5;
   rect.origin.y = ceilf( rect.origin.y ) + 0.5;
   rect.size.width =floorf( rect.size.width ) - 2.0;
   rect.size.height =floorf( rect.size.height ) - 2.0;
   rect = [self convertRectFromBase:rect]; 
   
   return rect;      
}

- (BOOL)wantsDefaultClipping { return NO; }

- (void)drawRect:(NSRect)rect {
   NSUInteger suit;
   NSUInteger value;
   NSUInteger index;
   
   NSUInteger count = [self.cards count];
   for( index = 0; index < (count - self.draggedCardsCount); index++ ) {
	   id card = [self.cards objectAtIndex:index];
      
      NSRect cardRect;
      cardRect = [self rectForCardWithIndex:index 
                                         count:count 
                                   getIsJammed:NULL];
      
      if( [self needsToDrawRect:cardRect] ) {
         BOOL selected 
         = [self.selectedCardsIndices containsIndex:index];
         [self drawCardBorderInRect:cardRect selected:selected];
         
         suit = [[card valueForKey:@"suit"] integerValue];
         value = [[card valueForKey:@"value"] integerValue];
         [self drawCardFaceWithSuit:suit 
                              value:value 
                             inRect:cardRect];
      }
   }    
   
   // Zeichne Auswahlrechteck 
   NSRect normRect = self.dragRect;
   if( normRect.size.width < 0.0 ) {
      normRect.origin.x += normRect.size.width;
      normRect.size.width *= -1.0;
   }
   if( normRect.size.height < 0.0 ) {
      normRect.origin.y += normRect.size.height;
      normRect.size.height *= -1.0;
   }
   
   if( !NSIsEmptyRect( normRect ) ) { 
      NSLog( @"Redraw während Selektierung" );
      // Rechteck auf View begrenzen 
      normRect = NSIntersectionRect( normRect, self.bounds );
   
      // Rechteck auf Pixel ausrichten 
      normRect = [self convertRectToBase:normRect]; 
      normRect = NSInsetRect( normRect, .5, .5 ); 
      normRect = [self convertRectFromBase:normRect];
      
      NSBezierPath* path;
      path = [NSBezierPath bezierPathWithRect:normRect];
      [path setLineWidth:0.0];
      
      [[NSColor colorWithCalibratedWhite:0.3 alpha:0.3] setFill];
      [path fill];
      [[NSColor blackColor] setFill];
      [path stroke];
   }
}

- (BOOL)acceptsFirstMouse:(NSEvent*)event { return YES; }

// Hilfsfunktion
- (NSInteger)indexOfCardAtLocation:(NSPoint)location 
{
   NSUInteger cardCount = [self.cards count];
   NSInteger cardIndex;
   for( cardIndex = cardCount-1; cardIndex > -1; cardIndex-- ) {
      NSRect cardRect = [self rectForCardWithIndex:cardIndex 
                                             count:cardCount
                                       getIsJammed:NULL];
      if( NSPointInRect( location, cardRect ) ){
         // gefunden
         break;
      }
   } 
   return cardIndex;
}

- (void)mouseDragged:(NSEvent*)event 
{
   // Bitte gehen Sie weiter, hier gibt es nichts zu sehen
   NSUInteger count = [self.cards count];
   if( count == 0) {
      return;
   }
   
   // Image erzeugen 
   //    gezogene Karte besorgen 
   NSPoint location = [event locationInWindow];
   location = [self convertPoint:location fromView:nil];
   NSInteger index = [self indexOfCardAtLocation:location];
   
   //    Falls ausserhalb des Stapels: Weg!
   if( index == -1 ) {
      return;
   }
   
   //	Groesse berechnen 
   NSRect draggedRect = [self rectForCardWithIndex:index
                                             count:count
                                       getIsJammed:NULL];
   NSRect lastRect = [self rectForCardWithIndex:count-1
                                          count:count
                                    getIsJammed:NULL];
   
   NSRect imageRect = NSUnionRect( draggedRect, lastRect );
   
   NSSize offset = NSMakeSize( imageRect.origin.x - 10.5, 
                              imageRect.origin.y - 10.5 );
   imageRect.size.width += 21.0;
   imageRect.size.height += 21.0;
   
   //    Bild zeichnen
   NSImage* dragImage = [[[NSImage alloc]
                          initWithSize:imageRect.size]
                         autorelease];
   [dragImage lockFocus];
   
   NSInteger suit;
   NSInteger value;
   NSInteger drawIndex;
   for( drawIndex = index; drawIndex < count; drawIndex++ ) {
      id card = [self.cards objectAtIndex:drawIndex];
      
      NSRect drawRect = [self rectForCardWithIndex:drawIndex
                                             count:count
                                       getIsJammed:NULL];
      drawRect = NSOffsetRect( drawRect, -offset.width,
                              -offset.height );
      [NSGraphicsContext saveGraphicsState];
      NSShadow* shadow = [[[NSShadow alloc] init] autorelease];
      [shadow setShadowBlurRadius:10.0];
      [shadow set];
      
      [self drawCardBorderInRect:drawRect selected:NO];
      [NSGraphicsContext restoreGraphicsState];
      
      suit = [[card valueForKey:@"suit"] integerValue];
      value = [[card valueForKey:@"value"] integerValue];
      
      [self drawCardFaceWithSuit:suit
                           value:value
                          inRect:drawRect];
   }                       
   
   [dragImage unlockFocus];
   
   // Pasteboard erstellen und befuellen
   NSPasteboard* pasteboard
   = [NSPasteboard pasteboardWithName:NSDragPboard];
   NSArray* dragTypes
   = [NSArray arrayWithObject:NSTIFFPboardType];
   [pasteboard declareTypes:dragTypes owner:nil];
   [pasteboard setData:[dragImage TIFFRepresentation]
               forType:NSTIFFPboardType];
   
   // Gezogene Karten ausblenden 
   self.draggedCardsCount = [self.cards count] - index;
   [self setNeedsDisplay:YES];
   
   // Ab dafuer
   imageRect.origin.x -= 10.5;
   imageRect.origin.y -= 10.5;
   
   [self  dragImage:dragImage
                 at:imageRect.origin
             offset:NSZeroSize
              event:event
         pasteboard:pasteboard
             source:self
          slideBack:YES];
   
   // Gezogene Karten wieder einfuegen
   self.draggedCardsCount = 0;
   [self setNeedsDisplay:YES];
}

- (BOOL)acceptsFirstResponder { return YES; }

- (BOOL)becomeFirstResponder
{
   if( [self.cards count] > 0 ) {
      self.selectedCardsIndices
      = [NSIndexSet indexSetWithIndex:[self.cards count]-1];
   }
   return YES;
}

- (BOOL)resignFirstResponder
{
   self.selectedCardsIndices = [NSIndexSet indexSet];
   return YES;
}

- (void)keyDown:(NSEvent*)event
{
   NSUInteger count = [self.cards count];
   
   // Keine Karten, keine Reaktion
   if( count == 0 ) {
      [[self nextResponder] keyDown:event];
      return;
   }
   
   NSUInteger selection = [self.selectedCardsIndices firstIndex];
   unichar character = [[event characters] characterAtIndex:0];
   switch( character ) {
      case NSUpArrowFunctionKey:
         if( selection > 0 ) {
            self.selectedCardsIndices 
            = [NSIndexSet indexSetWithIndex:selection-1];
         // Richtige Taste, aber bereits am Anschlag 
         } else {
            [super keyDown:event];
         } 
         break;
         
      case NSDownArrowFunctionKey:
         if( selection < (count-1) ) {
            self.selectedCardsIndices 
            = [NSIndexSet indexSetWithIndex:selection+1];
         // Richtige Taste, aber bereits am Anschlag
         } else {
            [super keyDown:event];
         } 
         break;
      
      // Nicht benutzte Taste
      default:
         [super keyDown:event];
         break;
   }
}

- (NSDragOperation)
   draggingSourceOperationMaskForLocal:(BOOL)local {
   NSLog( @"Operation fuer local %d", local );
   if( local ) {
      return	NSDragOperationMove
             | NSDragOperationCopy
             | NSDragOperationDelete;
   } else { 
      return NSDragOperationCopy;
   }
}

- (BOOL)ignoreModifierKeysWhileDragging
{
   return NO;
}

- (NSString*)operationText:(NSDragOperation)operation 
{
   NSString* outText = @"";
   if( operation & NSDragOperationCopy ) {
      outText = [outText stringByAppendingString:@"Copy " ];
   } 
   if( operation & NSDragOperationMove ) {
      outText = [outText stringByAppendingString:@"Move "];
   }
   if( operation & NSDragOperationDelete ) {
      outText = [outText stringByAppendingString:@"Delete "];
   }
   return outText;
}
   
- (NSDragOperation)draggingEntered:(id<NSDraggingInfo>)info
{
   NSDragOperation op = [info draggingSourceOperationMask];
   NSLog( @"Entered %@", [self operationText:op] );
   
   if( [self.cards count] > 0 ) { 
      self.selectedCardsIndices
      = [NSIndexSet indexSetWithIndex:[self.cards count]-1];
      [self setNeedsDisplay:YES];
   }
   
   if( op & NSDragOperationMove ) { 
      return NSDragOperationMove;
   } else if( op & NSDragOperationCopy ) {
      return NSDragOperationCopy;
   } else {
      return NSDragOperationNone;
   }
}

- (void)draggingExited:(id<NSDraggingInfo>)info 
{
   self.selectedCardsIndices = [NSIndexSet indexSet];

   [self setNeedsDisplay:YES];
   NSLog( @"exited constants %X",
          [info draggingSourceOperationMask] );
}

- (void)draggingEnded:(id<NSDraggingInfo>)info
{
   self.selectedCardsIndices = [NSIndexSet indexSet];
   NSDragOperation op = [info draggingSourceOperationMask];
   NSLog( @"Ended %@", [self operationText:op] );
}

- (BOOL)wantsPeriodicDraggingUpdates { return NO; } 
- (NSDragOperation)draggingUpdated:(id<NSDraggingInfo>)info
{
   NSDragOperation op = [info draggingSourceOperationMask];
   NSLog( @"Updated %@", [self operationText:op] );
   if( op & NSDragOperationMove ) {
      return NSDragOperationMove;
   } else if( op & NSDragOperationCopy ) {
      return NSDragOperationCopy;
   } else {
      return NSDragOperationNone;
   }
}

- (BOOL)prepareForDragOperation:(id<NSDraggingInfo>)info 
{
   NSDragOperation op = [info draggingSourceOperationMask];
   NSLog( @"prepare %@", [self operationText:op] );
   return YES;
}

- (BOOL)performDragOperation:(id<NSDraggingInfo>)info
{
   NSDragOperation op = [info draggingSourceOperationMask];
   NSLog( @"perform %@", [self operationText:op] );
   if((op & NSDragOperationMove) || (op & NSDragOperationCopy)) {
      return YES;
   } else {
      return NO;
   }
}

- (id)initWithFrame:(NSRect)frame {
   self = [super initWithFrame:frame];
   if (self) {
      self.alignment = NSMaxYEdge;
      
      NSDictionary* card;
      NSMutableArray* initCards = [NSMutableArray array];
      card = [NSDictionary dictionaryWithObjectsAndKeys:
              [NSNumber numberWithUnsignedInteger:1], @"suit",
              [NSNumber numberWithUnsignedInteger:1], @"value",
              [NSNumber numberWithBool:YES], @"disclosed",
              nil];
      [initCards addObject:card];
      card = [NSDictionary dictionaryWithObjectsAndKeys:
              [NSNumber numberWithUnsignedInteger:2], @"suit",
              [NSNumber numberWithUnsignedInteger:7], @"value",
              [NSNumber numberWithBool:YES], @"disclosed",
              nil];
      [initCards addObject:card];
      card = [NSDictionary dictionaryWithObjectsAndKeys:
              [NSNumber numberWithUnsignedInteger:2], @"suit",
              [NSNumber numberWithUnsignedInteger:7], @"value",
              [NSNumber numberWithBool:YES], @"disclosed",
              nil];
      [initCards addObject:card];
      self.cards = initCards;
      self.alignment = NSMaxXEdge;
      self.selectedCardsIndices = [NSIndexSet indexSet];
      
      // Drop-Types registrieren
      NSArray* dropTypes 
      = [NSArray arrayWithObject:NSTIFFPboardType];
      [self registerForDraggedTypes:dropTypes];
   }
   return self;
}

- (void) dealloc
{
   self.selectedCardsIndices = nil;
   self.cards = nil;
   self.alignment = 0;
   [super dealloc];
}

@end
