//  TableBaizeView.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "TableBaizeView.h"

#import "CardStackView.h"

const NSUInteger playerStacksMinIndex = 0;
const NSUInteger playerStacksCount = 10;
const NSUInteger depotStackIndex = 10;
const NSUInteger targetStackIndex = 11;

@implementation TableBaizeView
@synthesize dataSource;

- (CardStackView*)viewWithStackIndex:(NSUInteger)index
{
   if( index == depotStackIndex ) {
      return depotView;
   } else if( index == targetStackIndex ) {
      return targetView;
   } else {
      return [playerViews objectAtIndex:index];
   }
}

- (void)resizeSubviews
{
   NSSize cardSize;
   NSRect rect;
   NSRect bounds = [self bounds];
   
   // Kartenausmasse bestimmen
   CGFloat border = bounds.size.width / 111;
   cardSize.width = bounds.size.width / 11.1;
   cardSize.height
   = [[CardStackView class] cardHeightForWidth:cardSize.width];
   
   // Hieraus ergibt sich das Rechteck, in dem die Karten liegen
   bounds = NSInsetRect( bounds, border, border );
   
   rect.origin = bounds.origin;
   rect.size.width = NSWidth( bounds ) / 2;
   rect.size.height = cardSize.height;
   
   rect = [self centerScanRect:rect]; 
   
   [depotView setFrame:rect];
   
   rect.origin.x = NSMaxX( rect );
   
   rect = [self centerScanRect:rect];
   [targetView setFrame:rect];
   
   // Fuer die oberen Karten ergibt sich daraus das Rechteck
   rect.origin.x = NSMinX( bounds );
   rect.origin.y = NSMaxY( rect ) + border;
   rect.size.width = cardSize.width;
   rect.size.height = NSHeight( bounds) - NSMinY( rect );
   
   // die einzelnen Views einfach nacheinander setzen
   for( CardStackView* playerView in playerViews ) {
      rect = [self centerScanRect:rect];
      [playerView setFrame:rect];
      rect.origin.x += rect.size.width + border;
   }
}


- (void)resizeSubviewsWithOldSize:(NSSize)oldSize {
   [self resizeSubviews];
}

- (void)drawRect:(NSRect)rect {
   NSColor* lighterColor = [NSColor colorWithCalibratedRed:0.2 
                                                     green:0.7 
                                                      blue:0.2 
                                                     alpha:1.0];
   NSColor* darkerColor  = [NSColor colorWithCalibratedRed:0.0 
                                                     green:0.5 
                                                      blue:0.0 
                                                     alpha:1.0];
   NSGradient* gradient;
   gradient = [[[NSGradient alloc] 
                initWithStartingColor:lighterColor
                endingColor:darkerColor] 
               autorelease];
   
   [gradient             drawInRect:[self bounds]
             relativeCenterPosition:NSMakePoint( -0.5, +0.5 )]; 
}
#pragma mark -

#pragma mark Reloading
- (void)reloadDataForStackWithIndex:(NSUInteger)stackIndex {
   CardStackView* stackView; 
   stackView = [self viewWithStackIndex:stackIndex];
   
   id source = self.dataSource;
   if( !source ) {
      return;
   }
   
   id stack = [source tableBaizeView:self 
                      stackWithIndex:stackIndex];
   NSUInteger cardsCount = [source tableBaizeView:self 
                              countOfCardsOnStack:stack];
   NSMutableArray* cards 
   = [NSMutableArray arrayWithCapacity:cardsCount];
   
   NSUInteger cardIndex;
   NSDictionary* cardDictionary;
   for( cardIndex = 0; cardIndex < cardsCount; cardIndex++ ) {
      id card = [dataSource tableBaizeView:self
                             cardWithIndex:cardIndex 
                                   onStack:stack];
      NSUInteger skalar;
      skalar = [dataSource tableBaizeView:self suitOfCard:card];
      NSNumber* suit = [NSNumber numberWithInteger:skalar];
      skalar = [dataSource tableBaizeView:self valueOfCard:card];
      NSNumber* value = [NSNumber numberWithInteger:skalar];
      
      cardDictionary 
      = [NSDictionary dictionaryWithObjectsAndKeys:
         value, @"value",
         suit, @"suit",
         nil];
      
      [cards addObject:cardDictionary];
   }
   
   [stackView setCards:cards];
}

- (void)reloadData
{
   if( self.dataSource == nil ) {
      return;
   }
   
   // player stacks
   NSInteger playerStacksIndex;
   NSInteger index;
   for( index = 0; index < playerStacksCount; index++ ) {
      playerStacksIndex = playerStacksMinIndex + index;
      [self reloadDataForStackWithIndex:playerStacksIndex];
   }
   
   [self reloadDataForStackWithIndex:depotStackIndex];
   [self reloadDataForStackWithIndex:targetStackIndex];
}
#pragma mark -

#pragma mark Creation
- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
       // Zehn obere Stapel
       CardStackView* view;
       playerViews = [[NSMutableArray alloc] init];
       NSUInteger index;
       for( index = 0; index < 10; index++ ) {
          view = [[[CardStackView alloc] 
                   initWithFrame:NSZeroRect] 
                  autorelease];
          [view setAlignment:NSMaxYEdge];
          [self addSubview:view];
          [playerViews addObject:view];
       }
       
       // Depot
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMinXEdge];
       [self addSubview:view];
       depotView = view;
       
       // Ablage
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMaxXEdge];
       [self addSubview:view];
       targetView = view;
       
       [self resizeSubviews];
    }
    return self;
}

- (void) dealloc
{
   [playerViews release];
   playerViews = nil;
   depotView = nil;
   targetView = nil;
   
   [super dealloc];
}
@end
