//  TableBaizeView.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "TableBaizeView.h"

#import "CardStackView.h"

const NSUInteger playerStacksMinIndex = 0;
const NSUInteger playerStacksCount = 10;
const NSUInteger depotStackIndex = 10;
const NSUInteger targetStackIndex = 11;

@implementation TableBaizeView
- (NSColor*)color { return color; }
- (void)setColor:(NSColor*)value 
{
   if( value != color ) {
      [color release];
      color = [value copy];
      [self setNeedsDisplay:YES];
   }
}

- (CardStackView*)viewWithStackIndex:(NSUInteger)index
{
   if( index == depotStackIndex ) {
      return depotView;
   } else if( index == targetStackIndex ) {
      return targetView;
   } else {
      return [playerViews objectAtIndex:index];
   }
}

- (void)resizeSubviews
{
   NSSize cardSize;
   NSRect rect;
   NSRect bounds = [self bounds];
   
   // Kartenausmasse bestimmen
   CGFloat border = bounds.size.width / 111;
   cardSize.width = bounds.size.width / 11.1;
   cardSize.height
   = [[CardStackView class] cardHeightForWidth:cardSize.width];
   
   // Hieraus ergibt sich das Rechteck, in dem die Karten liegen
   bounds = NSInsetRect( bounds, border, border );
   
   rect.origin = bounds.origin;
   rect.size.width = NSWidth( bounds ) / 2;
   rect.size.height = cardSize.height;
   
   rect = [self centerScanRect:rect]; 
   
   [depotView setFrame:rect];
   
   rect.origin.x = NSMaxX( rect );
   
   rect = [self centerScanRect:rect];
   [targetView setFrame:rect];
   
   // Fuer die oberen Karten ergibt sich daraus das Rechteck
   rect.origin.x = NSMinX( bounds );
   rect.origin.y = NSMaxY( rect ) + border;
   rect.size.width = cardSize.width;
   rect.size.height = NSHeight( bounds) - NSMinY( rect );
   
   // die einzelnen Views einfach nacheinander setzen
   for( CardStackView* playerView in playerViews ) {
      rect = [self centerScanRect:rect];
      [playerView setFrame:rect];
      rect.origin.x += rect.size.width + border;
   }
}


- (void)resizeSubviewsWithOldSize:(NSSize)oldSize {
   [self resizeSubviews];
}

- (void)drawRect:(NSRect)rect {
   NSColor* darkerColor = self.color;
   NSColor* lighterColor = [darkerColor highlightWithLevel:0.7];
   
   NSGradient* gradient;
   gradient = [[[NSGradient alloc] 
                initWithStartingColor:lighterColor
                endingColor:darkerColor] 
               autorelease];
   
   [gradient             drawInRect:[self bounds]
             relativeCenterPosition:NSMakePoint( -0.5, +0.5 )]; 
}
#pragma mark -

#pragma mark Key-Value Observing
- (void)observeValueForKeyPath:(NSString*)keyPath 
                      ofObject:observed 
                        change:(NSDictionary*)change 
                       context:(id)context
{
   // Ist es meine Observierung?
   if( [context isEqualToString:@"colorBinding"] ) {
      NSLog( @"stored: %@", self.color );
      NSLog( @"change: %@", change );

      // neuen Wert abholen und setzen.
      self.color = [change objectForKey:NSKeyValueChangeNewKey];
      return;
   } else {
      [super observeValueForKeyPath:keyPath 
                           ofObject:observed 
                             change:change 
                            context:context];
   }
}
#pragma mark -

#pragma mark Creation
- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
       self.color = [NSColor greenColor];
       
       // Zehn obere Stapel
       CardStackView* view;
       playerViews = [[NSMutableArray alloc] init];
       NSUInteger index;
       for( index = 0; index < 10; index++ ) {
          view = [[[CardStackView alloc] 
                   initWithFrame:NSZeroRect] 
                  autorelease];
          [view setAlignment:NSMaxYEdge];
          [self addSubview:view];
          [playerViews addObject:view];
       }
       
       // Depot
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMinXEdge];
       [self addSubview:view];
       depotView = view;
       
       // Ablage
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMaxXEdge];
       [self addSubview:view];
       targetView = view;
       
       [self resizeSubviews];
    }
    return self;
}

- (void) dealloc
{
   self.color = nil;
   [playerViews release];
   playerViews = nil;
   depotView = nil;
   targetView = nil;
   
   [super dealloc];
}
@end
