//  AppDelegate.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "AppDelegate.h"

#import "TableBaizeView.h"
#import "CardStackView.h"
#import "Card.h";
#import "CardStack.h"

@implementation AppDelegate
@synthesize stacks;

- (IBAction)addCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   Card* card = [[[Card alloc] init] autorelease];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   card.value = [cards count] + 1;
   [cards addObject:card];
}

- (IBAction)removeLastCard:(id)sender { 
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 0 ) {
      [cards removeObject:[cards lastObject]];
   }
}

- (IBAction)replaceFirstCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 0 ) {
      Card* card = [[[Card alloc] init] autorelease];
      card.value = 12;
      [cards replaceObjectAtIndex:0 withObject:card];
   }
}

- (IBAction)replaceFirstAndLastCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 1 ) {
      Card* card1 = [[[Card alloc] init] autorelease];
      card1.value = 4;
      Card* card2 = [[[Card alloc] init] autorelease];
      card2.value = 6;
      NSArray* replaceCards 
      = [NSArray arrayWithObjects:card1, card2, nil];
      NSMutableIndexSet* indexes = [NSMutableIndexSet indexSet];
      [indexes addIndex:[cards count]-1];
      [indexes addIndex:0];
      [cards replaceObjectsAtIndexes:indexes 
                         withObjects:replaceCards];
   }
}

- (IBAction)setStack:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* newCards = [NSMutableArray array];
   NSUInteger value;
   Card* card;
   for( value = 1; value < 7; value++ ) {
      card = [[[Card alloc] init] autorelease];
      card.value = value;
      [newCards addObject:card];
   }
   stack.cards = newCards;
}

- (NSMutableArray*)cardsHeap 
{
   NSUInteger setIndex;
   NSUInteger suit;
   NSUInteger value;
   
   NSMutableArray* heap = [NSMutableArray arrayWithCapacity:104];
   for( setIndex = 0; setIndex < 4; setIndex++ ) {
      for( suit = 1; suit < 3; suit++ ) {
         for ( value = 1; value < 14; value++ ) {
            Card* card = [[[Card alloc] init] autorelease];
            card.value = value;
            card.suit = [NSNumber numberWithInteger:suit];
            card.disclosed = NO;
            [heap addObject:card];
         }
      }
   }
   
   return heap;
}

- (void)shuffleCardsHeap:(NSMutableArray*)heap {
   // mischen
   srandom( 9811 );
   NSUInteger count = 10000;
   while( count-- > 0 ) {
      NSUInteger swapIndex = (float)random() / INT32_MAX * 104;
      [heap insertObject:[heap lastObject] atIndex:swapIndex];
      [heap removeLastObject];
   }
}

- (NSArray*)stacksWithDealtCards:(NSMutableArray*)heap {
   NSUInteger stackIndex;
   NSMutableArray* newStacks;
   CardStack* stack;
   NSMutableArray* cards;
   
   // Stapel erzeugen
   newStacks = [NSMutableArray arrayWithCapacity:12];
   for( stackIndex = 0; stackIndex < 12; stackIndex++ ) {
      CardStack* stack = [[[CardStack alloc] init] autorelease];
      [newStacks addObject:stack];
   }
   
   //    50 Karten auf das Depot, im Model Stapel mit Index 10
   stack = [newStacks objectAtIndex:10];
   cards = [NSMutableArray arrayWithCapacity:50];
   NSUInteger cardIndex;
   for( cardIndex = 0; cardIndex < 50; cardIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      [cards addObject:card];
   }
   stack.cards = cards;
   
   // die restlichen Karten bis auf 10 Karten verteilen
   stackIndex = 0;
   while( [heap count] > 10 ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      
      CardStack* stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];
      
      if( stackIndex == 9 ) {
         stackIndex = 0;
      } else {
         stackIndex++;
      }
   }
   
   //  Die letzten 10 Karten aufgedeckt verteilen
   for(stackIndex = 0; stackIndex < 10; stackIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      card.disclosed = YES;
      
      stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];   
   }
   
   return newStacks;
}

- (void)awakeFromNib 
{
   CardStack* targetStack 
      = [self.stacks objectAtIndex:targetStackIndex];
   
   NSView* targetView 
      = [tableBaizeView viewWithStackIndex:targetStackIndex];
   
   NSKeyValueObservingOptions options;
   options =   NSKeyValueObservingOptionNew
   | NSKeyValueObservingOptionOld
   | NSKeyValueObservingOptionInitial;

   [targetStack addObserver:targetView 
                 forKeyPath:@"cards" 
                    options:options
                    context:@"cardsBinding"];
}

- (id)init
{
   self = [super init];
   if( self ) {
      // Karten erzeugen, mischen und austeilen
      NSMutableArray* heap = [self cardsHeap];
      [self shuffleCardsHeap:heap];
      self.stacks = [self stacksWithDealtCards:heap];
   }
   return self;
}

- (void) dealloc
{
   CardStackView* targetView 
   = [tableBaizeView viewWithStackIndex:targetStackIndex];
   CardStack* targetStack = [self.stacks objectAtIndex:11];
   [targetStack removeObserver:targetView forKeyPath:@"cards"];
   
   self.stacks = nil;
   [super dealloc];
}
@end
