//  AppDelegate.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "AppDelegate.h"

#import "TableBaizeView.h"
#import "Card.h";
#import "CardStack.h"

#import "CompressArrayValueTransformer.h"
#import "DisclosedFilterArrayController.h";

@implementation AppDelegate
@synthesize stacks;
@synthesize filterControllers;

- (IBAction)addCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   Card* card = [[[Card alloc] init] autorelease];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   card.value = [cards count] + 1;
   [cards addObject:card];
}

- (IBAction)removeLastCard:(id)sender { 
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 0 ) {
      [cards removeObject:[cards lastObject]];
   }
}

- (IBAction)replaceFirstCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 0 ) {
      Card* card = [[[Card alloc] init] autorelease];
      card.value = 12;
      [cards replaceObjectAtIndex:0 withObject:card];
   }
}

- (IBAction)replaceFirstAndLastCard:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 1 ) {
      Card* card1 = [[[Card alloc] init] autorelease];
      card1.value = 4;
      Card* card2 = [[[Card alloc] init] autorelease];
      card2.value = 6;
      NSArray* replaceCards 
      = [NSArray arrayWithObjects:card1, card2, nil];
      NSMutableIndexSet* indexes = [NSMutableIndexSet indexSet];
      [indexes addIndex:[cards count]-1];
      [indexes addIndex:0];
      [cards replaceObjectsAtIndexes:indexes 
                         withObjects:replaceCards];
   }
}

- (IBAction)changeLastCard:(id)sender
{
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* cards 
   = [stack mutableArrayValueForKey:@"cards"];
   if( [cards count] > 0 ) {
      Card* card = [cards lastObject];
      card.suit = [NSNumber numberWithUnsignedInteger:2];
   }
}

- (IBAction)setStack:(id)sender {
   CardStack* stack = [self.stacks objectAtIndex:11];
   NSMutableArray* newCards = [NSMutableArray array];
   NSUInteger value;
   Card* card;
   for( value = 1; value < 7; value++ ) {
      card = [[[Card alloc] init] autorelease];
      card.value = value;
      [newCards addObject:card];
   }
   stack.cards = newCards;
}

- (NSMutableArray*)cardsHeap 
{
   NSUInteger setIndex;
   NSUInteger suit;
   NSUInteger value;
   
   NSMutableArray* heap = [NSMutableArray arrayWithCapacity:104];
   for( setIndex = 0; setIndex < 4; setIndex++ ) {
      for( suit = 1; suit < 3; suit++ ) {
         for ( value = 1; value < 14; value++ ) {
            Card* card = [[[Card alloc] init] autorelease];
            card.value = value;
            card.suit = [NSNumber numberWithInteger:suit];
            card.disclosed = NO;
            [heap addObject:card];
         }
      }
   }
   
   return heap;
}

- (void)shuffleCardsHeap:(NSMutableArray*)heap {
   // mischen
   srandom( 9811 );
   NSUInteger count = 10000;
   while( count-- > 0 ) {
      NSUInteger swapIndex = (float)random() / INT32_MAX * 104;
      [heap insertObject:[heap lastObject] atIndex:swapIndex];
      [heap removeLastObject];
   }
}

- (NSArray*)stacksWithDealtCards:(NSMutableArray*)heap {
   NSUInteger stackIndex;
   NSMutableArray* newStacks;
   CardStack* stack;
   NSMutableArray* cards;
   
   // Stapel erzeugen
   newStacks = [NSMutableArray arrayWithCapacity:12];
   for( stackIndex = 0; stackIndex < 12; stackIndex++ ) {
      CardStack* stack = [[[CardStack alloc] init] autorelease];
      [newStacks addObject:stack];
   }
   
   //    50 Karten auf das Depot, im Model Stapel mit Index 10
   stack = [newStacks objectAtIndex:10];
   cards = [NSMutableArray arrayWithCapacity:50];
   NSUInteger cardIndex;
   for( cardIndex = 0; cardIndex < 50; cardIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      [cards addObject:card];
   }
   stack.cards = cards;
   
   // die restlichen Karten bis auf 10 Karten verteilen
   stackIndex = 0;
   while( [heap count] > 10 ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      
      CardStack* stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];
      
      if( stackIndex == 9 ) {
         stackIndex = 0;
      } else {
         stackIndex++;
      }
   }
   
   //  Die letzten 10 Karten aufgedeckt verteilen
   for(stackIndex = 0; stackIndex < 10; stackIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      card.disclosed = YES;
      
      stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];   
   }
   
   return newStacks;
}

- (IBAction)discloseCards:(id)sender
{
   CardStack* stack;
   stack = [self.stacks objectAtIndex:playerStacksMinIndex];
   for( Card* card in stack.cards ) {
      card.disclosed = YES;
   }
}

- (void)awakeFromNib 
{
   [tableBaizeView setValueKeyPath:@"value"
                    andSuitKeyPath:@"suit"];
   
   CompressArrayValueTransformer* transformer;
   transformer = [[[CompressArrayValueTransformer alloc]
                   init]
                  autorelease];
   transformer.compression = 3;
   
   NSDictionary* options
   = [NSDictionary dictionaryWithObject:transformer 
                   forKey:NSValueTransformerBindingOption];
   
   CardStack* targetStack = [self.stacks objectAtIndex:11];
   [tableBaizeView bind:targetDisclosedCardsBinding
               toObject:targetStack
            withKeyPath:@"cards"
                options:options];
   
   CardStack* depotStack 
   = [self.stacks objectAtIndex:depotStackIndex];
   [tableBaizeView bind:depotDisclosedCardsBinding
               toObject:depotStack
            withKeyPath:@"cards"
                options:nil];
   
   NSUInteger index;
   NSMutableArray* controllers = [NSMutableArray array];
   NSPredicate* filter;
   filter = [NSPredicate predicateWithFormat:@"disclosed==YES"];
   for( index = 0; index < playerStacksCount; index++ ) {
      NSUInteger stackIndex = index + playerStacksMinIndex;
      
      // Controller ...
      NSArrayController* controller;
      controller =[[[DisclosedFilterArrayController alloc] 
                                                    init] 
                                                    autorelease];
      [controllers addObject:controller];
      [controller setAutomaticallyRearrangesObjects:YES];   
      [controller setFilterPredicate:filter];
      
      // ... und dessen Binding an das Model
      CardStack* stack = [self.stacks objectAtIndex:stackIndex];
      [controller bind:NSContentArrayBinding
              toObject:stack
           withKeyPath:@"cards"
               options:nil];
      
      // Bindung des Views an den Controller
      NSString* binding = [NSString stringWithFormat:@"%@%d",
                                   playerNDisclosedCardsBinding, 
                                   stackIndex];
      [tableBaizeView bind:binding
                  toObject:controller
               withKeyPath:@"arrangedObjects"
                   options:nil];
      
      // Bindung des Views an den Controller Non-Disclosed-Count
      binding = [NSString stringWithFormat:@"%@%d",
                 playersNonDisclosedCardsCountBinding, 
                 stackIndex];
      [tableBaizeView bind:binding
                  toObject:controller
               withKeyPath:@"filteredOutCount"
                   options:nil];   
      
   }
   
}

- (id)init
{
   self = [super init];
   if( self ) {
      // Karten erzeugen, mischen und austeilen
      NSMutableArray* heap = [self cardsHeap];
      [self shuffleCardsHeap:heap];
      self.stacks = [self stacksWithDealtCards:heap];
   }
   return self;
}

- (void)applicationWillTerminate:(NSNotification*)notification
{
   [tableBaizeView unbind:targetDisclosedCardsBinding];
}

- (void) dealloc
{
   self.stacks = nil;
   [super dealloc];
}
@end
