//  AppDelegate.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "AppDelegate.h"

#import "TableBaizeView.h"
#import "Card.h";
#import "CardStack.h"
#import "DepotStack.h"

#import "CompressArrayValueTransformer.h"
#import "DisclosedFilterArrayController.h";

@implementation AppDelegate
@synthesize stacks;
@synthesize filterControllers;

#pragma mark Delegate
- (BOOL)        tableBaizeView:(TableBaizeView*)view
        shouldDragCardsAtIndex:(NSUInteger)cardIndex
            fromStackWithIndex:(NSUInteger)stackIndex
{
   // Hole Stapel
   // Es muss sich um einen Spielerstapel handeln
   stackIndex += playerStacksMinIndex;
   CardStack* stack = [[self stacks] objectAtIndex:stackIndex];
   
   // Alle Karten ab der gezogenen müssen aufeinander folgen
   Card* card = [[stack cards] objectAtIndex:cardIndex];
   NSNumber* suit = card.suit;
   NSUInteger value = card.value;
   for( cardIndex++; 
       cardIndex < [stack.cards count]; 
       cardIndex++ ) 
   {
      value--;
      card = [[stack cards] objectAtIndex:cardIndex];
      if(    ![card.suit isEqualToNumber:suit] 
         || (card.value != value) ) 
      {
         break;
      }
   }
   
   // Schleife durchlaufen?
   return (cardIndex == [stack.cards count]);
}
#pragma mark -

#pragma mark Creation
- (void)tableBaizeViewDealCards:(TableBaizeView*)view 
{
   NSUInteger stackIndex;
   NSUInteger counter;
   
   // Depotkarten holen
   CardStack* depotStack 
   = [[self stacks] objectAtIndex:depotStackIndex];
   NSMutableArray* depotCards;
   depotCards = [depotStack mutableArrayValueForKey:@"cards"];
   
   for( counter = 0; counter < playerStacksCount; counter++ ) {
      stackIndex = counter + playerStacksMinIndex;
      
      // Spielerkarten holen
      CardStack* playerStack;
      playerStack = [[self stacks] objectAtIndex:stackIndex];
      
      NSMutableArray* playerCards;
      playerCards 
      = [playerStack mutableArrayValueForKey:@"cards"];
      
      // Aufdecken und Verschieben
      Card* card = [depotCards lastObject];
      card.disclosed = YES;
      [depotCards removeLastObject];
      
      [playerCards addObject:card];
   }
}

- (NSMutableArray*)cardsHeap 
{
   NSUInteger setIndex;
   NSUInteger suit;
   NSUInteger value;
   
   NSMutableArray* heap = [NSMutableArray arrayWithCapacity:104];
   for( setIndex = 0; setIndex < 4; setIndex++ ) {
      for( suit = 1; suit < 3; suit++ ) {
         for ( value = 1; value < 14; value++ ) {
            Card* card = [[[Card alloc] init] autorelease];
            card.value = value;
            card.suit = [NSNumber numberWithInteger:suit];
            card.disclosed = NO;
            [heap addObject:card];
         }
      }
   }
   
   return heap;
}

- (void)shuffleCardsHeap:(NSMutableArray*)heap {
   // mischen
   srandom( 9811 );
   NSUInteger count = 10000;
   while( count-- > 0 ) {
      NSUInteger swapIndex = (float)random() / INT32_MAX * 104;
      [heap insertObject:[heap lastObject] atIndex:swapIndex];
      [heap removeLastObject];
   }
}

- (NSArray*)stacksWithDealtCards:(NSMutableArray*)heap {
   NSUInteger stackIndex;
   NSMutableArray* newStacks;
   CardStack* stack;
   NSMutableArray* cards;
   
   // Stapel erzeugen
   newStacks = [NSMutableArray arrayWithCapacity:12];
   for( stackIndex = 0; stackIndex < 12; stackIndex++ ) {
      CardStack* stack;
      if( stackIndex == depotStackIndex ) {
         stack = [[[DepotStack alloc] init] autorelease];
      } else {
         stack = [[[CardStack alloc] init] autorelease];
      }
      [newStacks addObject:stack];
   }
   
   //    50 Karten auf das Depot, im Model Stapel mit Index 10
   stack = [newStacks objectAtIndex:10];
   cards = [NSMutableArray arrayWithCapacity:50];
   NSUInteger cardIndex;
   for( cardIndex = 0; cardIndex < 50; cardIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      [cards addObject:card];
   }
   stack.cards = cards;
   
   // die restlichen Karten bis auf 10 Karten verteilen
   stackIndex = 0;
   while( [heap count] > 10 ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      
      CardStack* stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];
      
      if( stackIndex == 9 ) {
         stackIndex = 0;
      } else {
         stackIndex++;
      }
   }
   
   //  Die letzten 10 Karten aufgedeckt verteilen
   for(stackIndex = 0; stackIndex < 10; stackIndex++ ) {
      Card* card = [heap lastObject];
      [heap removeLastObject];
      card.disclosed = YES;
      
      stack = [newStacks objectAtIndex:stackIndex];
      cards = [stack mutableArrayValueForKey:@"cards"];
      [cards addObject:card];   
   }
   
   return newStacks;
}

- (void)awakeFromNib 
{
   [tableBaizeView setDelegate:self];
   [tableBaizeView setValueKeyPath:@"value"
                    andSuitKeyPath:@"suit"];
   
   CompressArrayValueTransformer* transformer;
   transformer = [[[CompressArrayValueTransformer alloc]
                   init]
                  autorelease];
   transformer.compression = 3;
   
   NSDictionary* options
   = [NSDictionary dictionaryWithObject:transformer 
                   forKey:NSValueTransformerBindingOption];
   
   CardStack* targetStack = [self.stacks objectAtIndex:11];
   [tableBaizeView bind:targetDisclosedCardsBinding
               toObject:targetStack
            withKeyPath:@"cards"
                options:options];
   
   // Depotstack binden
   // Controller
   NSMutableArray* controllers = [NSMutableArray array];
   NSPredicate* filter;
   filter = [NSPredicate predicateWithFormat:@"disclosed==YES"];
   NSArrayController* controller;
   controller =[[[DisclosedFilterArrayController alloc] 
                 init] 
                autorelease];
   [controller setAutomaticallyRearrangesObjects:YES];
   [controllers addObject:controller];
   [controller setFilterPredicate:filter];
   
   // Controller an Stack binden   
   CardStack* depotStack 
      = [self.stacks objectAtIndex:depotStackIndex];
   [controller bind:NSContentArrayBinding
           toObject:depotStack
        withKeyPath:@"cards"
            options:nil];
   
   // Bindung des Depotviews an den Controller Karten
   [tableBaizeView bind:depotDisclosedCardsBinding
               toObject:controller
            withKeyPath:@"arrangedObjects"
                options:nil];
   
   // ... und Anzahl
   [tableBaizeView bind:depotNonDisclosedCardsCountBinding
               toObject:depotStack
            withKeyPath:@"count10OfNonDisclosedCards"
                options:nil];
   
   NSUInteger index;
   filter = [NSPredicate predicateWithFormat:@"disclosed==YES"];
   for( index = 0; index < playerStacksCount; index++ ) {
      NSUInteger stackIndex = index + playerStacksMinIndex;
      
      // Controller ...
      NSArrayController* controller;
      controller =[[[DisclosedFilterArrayController alloc] 
                                                    init] 
                                                    autorelease];
      [controllers addObject:controller];
      [controller setAutomaticallyRearrangesObjects:YES];   
      [controller setFilterPredicate:filter];
      
      // ... und dessen Binding an das Model
      CardStack* stack = [self.stacks objectAtIndex:stackIndex];
      [controller bind:NSContentArrayBinding
              toObject:stack
           withKeyPath:@"cards"
               options:nil];
      
      // Bindung des Views an den Controller
      NSString* binding = [NSString stringWithFormat:@"%@%d",
                                   playerNDisclosedCardsBinding, 
                                   stackIndex];
      [tableBaizeView bind:binding
                  toObject:controller
               withKeyPath:@"arrangedObjects"
                   options:nil];
      
      // Bindung des Views an den Controller Non-Disclosed-Count
      binding = [NSString stringWithFormat:@"%@%d",
                 playersNonDisclosedCardsCountBinding, 
                 stackIndex];
      [tableBaizeView bind:binding
                  toObject:controller
               withKeyPath:@"filteredOutCount"
                   options:nil];   
      
   }
   
}

- (id)init
{
   self = [super init];
   if( self ) {
      // Karten erzeugen, mischen und austeilen
      NSMutableArray* heap = [self cardsHeap];
      [self shuffleCardsHeap:heap];
      self.stacks = [self stacksWithDealtCards:heap];
   }
   return self;
}

- (void)applicationWillTerminate:(NSNotification*)notification
{
   [tableBaizeView unbind:targetDisclosedCardsBinding];
}

- (void) dealloc
{
   self.stacks = nil;
   [super dealloc];
}
@end
