//  TableBaizeView.m
//  Card Game
//
//  Created by Amin Negm-Awad
//  Copyright 2009 [co coa:ding]. All rights reserved.

#import "TableBaizeView.h"

#import "CardStackView.h"

const NSUInteger playerStacksMinIndex = 0;
const NSUInteger playerStacksCount = 10;
const NSUInteger depotStackIndex = 10;
const NSUInteger targetStackIndex = 11;

NSString* const targetDisclosedCardsBinding 
   = @"targetDisclosedCardsBinding";
NSString* const depotDisclosedCardsBinding 
   = @"depotDisclosedCardsBinding";
NSString* const depotNonDisclosedCardsCountBinding
   = @"depotNonDisclosedCardsCountBinding";
NSString* const playerNDisclosedCardsBinding 
   = @"playerNDisclosedCardsBinding";
NSString* const playersNonDisclosedCardsCountBinding
= @"playersNonDisclosedCardsCountBinding";

@implementation TableBaizeView
@synthesize delegate;
- (NSUInteger)indexOfView:(CardStackView*)view
{
   if( view == depotView ) {
      return depotStackIndex;
   } else if( view == targetView ) {
      return targetStackIndex;
   } else {
      NSUInteger index = [playerViews indexOfObject:view];
      if( index != NSNotFound ) {
         return index;
      }
   }
   
   return NSNotFound;
}

- (CardStackView*)viewWithStackIndex:(NSUInteger)index
{
   if( index == depotStackIndex ) {
      return depotView;
   } else if( index == targetStackIndex ) {
      return targetView;
   } else {
      return [playerViews objectAtIndex:index];
   }
}

- (void)resizeSubviews
{
   NSSize cardSize;
   NSRect rect;
   NSRect bounds = [self bounds];
   
   // Kartenausmasse bestimmen
   CGFloat border = bounds.size.width / 111;
   cardSize.width = bounds.size.width / 11.1;
   cardSize.height
   = [[CardStackView class] cardHeightForWidth:cardSize.width];
   
   // Hieraus ergibt sich das Rechteck, in dem die Karten liegen
   bounds = NSInsetRect( bounds, border, border );
   
   rect.origin = bounds.origin;
   rect.size.width = NSWidth( bounds ) / 2;
   rect.size.height = cardSize.height;
   
   rect = [self centerScanRect:rect]; 
   
   [depotView setFrame:rect];
   
   rect.origin.x = NSMaxX( rect );
   
   rect = [self centerScanRect:rect];
   [targetView setFrame:rect];
   
   // Fuer die oberen Karten ergibt sich daraus das Rechteck
   rect.origin.x = NSMinX( bounds );
   rect.origin.y = NSMaxY( rect ) + border;
   rect.size.width = cardSize.width;
   rect.size.height = NSHeight( bounds) - NSMinY( rect );
   
   // die einzelnen Views einfach nacheinander setzen
   for( CardStackView* playerView in playerViews ) {
      rect = [self centerScanRect:rect];
      [playerView setFrame:rect];
      rect.origin.x += rect.size.width + border;
   }
}


- (void)resizeSubviewsWithOldSize:(NSSize)oldSize {
   [self resizeSubviews];
}

- (void)drawRect:(NSRect)rect {
   NSColor* darkerColor = [NSColor greenColor];
   NSColor* lighterColor = [darkerColor highlightWithLevel:0.7];
   
   NSGradient* gradient;
   gradient = [[[NSGradient alloc] 
                initWithStartingColor:lighterColor
                endingColor:darkerColor] 
               autorelease];
   
   [gradient             drawInRect:[self bounds]
             relativeCenterPosition:NSMakePoint( -0.5, +0.5 )]; 
}
#pragma mark -

#pragma mark Bindings
- (void)setValueKeyPath:(NSString*)valueKeyPath 
         andSuitKeyPath:(NSString*)suitKeyPath 
{
   for( CardStackView* stackView in [self subviews] ) {
      stackView.valueKeyPath = valueKeyPath;
      stackView.suitKeyPath  = suitKeyPath;
   }
}

- (void)       bind:(NSString*)binding 
           toObject:(id)observed 
        withKeyPath:(NSString*)keyPath 
            options:(NSDictionary*)options
{
   CardStackView* stackView;
   if( binding == targetDisclosedCardsBinding ) {
      stackView = [self viewWithStackIndex:targetStackIndex];
      [stackView bind:disclosedCardsBinding
             toObject:observed
          withKeyPath:keyPath
              options:options];
      
   } else if( binding == depotDisclosedCardsBinding ) {
      stackView = [self viewWithStackIndex:depotStackIndex];
      [stackView bind:disclosedCardsBinding
             toObject:observed
          withKeyPath:keyPath
              options:options];      

   } else if( binding == depotNonDisclosedCardsCountBinding ) {
      stackView = [self viewWithStackIndex:depotStackIndex];
      [stackView bind:nonDisclosedCardsCountBinding
             toObject:observed
          withKeyPath:keyPath
              options:options];
      
   } else if( [binding hasPrefix:playerNDisclosedCardsBinding] ){
      NSUInteger indexPosition;
      indexPosition = [playerNDisclosedCardsBinding length];
      NSUInteger stackIndex
         = [[binding substringFromIndex:indexPosition] intValue];
      CardStackView* stackView;
      stackView = [self viewWithStackIndex:stackIndex];
      [stackView bind:disclosedCardsBinding
             toObject:observed
          withKeyPath:keyPath
              options:options];       

   } else if( [binding 
               hasPrefix:playersNonDisclosedCardsCountBinding] ){
      NSUInteger indexPosition;
      indexPosition 
      = [playersNonDisclosedCardsCountBinding length];
      NSUInteger stackIndex
      = [[binding substringFromIndex:indexPosition] intValue];
      CardStackView* stackView;
      stackView = [self viewWithStackIndex:stackIndex];
      [stackView bind:nonDisclosedCardsCountBinding
             toObject:observed
          withKeyPath:keyPath
              options:options];         
      
   } else {
      [super        bind:binding 
                toObject:observed 
             withKeyPath: keyPath
                 options:options];
   }
}

- (void)unbind:(NSString*)binding
{
   CardStackView* stackView;
   if( binding == targetDisclosedCardsBinding ) {
      stackView = [self viewWithStackIndex:targetStackIndex];
      [stackView unbind:disclosedCardsBinding];
      
   } else if( binding == depotDisclosedCardsBinding ) {
      stackView = [self viewWithStackIndex:depotStackIndex];
      [stackView unbind:disclosedCardsBinding];

   } else if( binding == depotNonDisclosedCardsCountBinding ) {
      stackView = [self viewWithStackIndex:depotStackIndex];
      [stackView unbind:nonDisclosedCardsCountBinding];
      
   } else if( [binding hasPrefix:playerNDisclosedCardsBinding] ){
      NSUInteger indexPosition;
      indexPosition = [playerNDisclosedCardsBinding length];
      NSUInteger stackIndex
      = [[binding substringFromIndex:indexPosition] intValue];
      CardStackView* stackView;
      stackView = [self viewWithStackIndex:stackIndex];
      [stackView unbind:disclosedCardsBinding];
      
   } else if( [binding 
               hasPrefix:playersNonDisclosedCardsCountBinding] ){
      NSUInteger indexPosition;
      indexPosition 
         = [playersNonDisclosedCardsCountBinding length];
      NSUInteger stackIndex
      = [[binding substringFromIndex:indexPosition] intValue];
      CardStackView* stackView;
      stackView = [self viewWithStackIndex:stackIndex];
      [stackView unbind:nonDisclosedCardsCountBinding];
      
   } else {
      [super unbind:binding];
   }   
}
#pragma mark -

#pragma mark Events
- (void)mouseDown:(NSEvent*)event {
   
   // Auf welchen Stapel wurde geklickt?
   NSPoint location = [event locationInWindow];
   location = [[self superview] convertPoint:location 
                                    fromView:nil]; 
   NSView* view = [self hitTest:location];
   if( view == self ) {
      return;
   }
   CardStackView* stackView = (CardStackView*)view;
   location = [stackView convertPoint:location
                             fromView:[self superview]];
   // Wurde auf die letzte Karte geklickt?
   NSUInteger cardsCount = [[stackView cards] count];
   cardsCount += [stackView countOfNonDisclosedCards];
   if(  cardsCount == 0 ) {
      return;
   }
   
   NSUInteger cardIndex;
   cardIndex = [stackView indexOfCardAtLocation:location];
   
   if( cardIndex != (cardsCount - 1) ) {
      return;
   }
   
   // Im Falle des Depotstapels, die Delegatemethode ausfuehren
   if( stackView == depotView ) {
      id myDelegate = self.delegate;
      SEL dealSelector = @selector( tableBaizeViewDealCards: );
      if( [myDelegate respondsToSelector:dealSelector] ) {
         [myDelegate tableBaizeViewDealCards:self];
      }
      
   }
}
#pragma mark -

#pragma mark Delegating
- (BOOL)allowsDraggingFromCardStackView:(CardStackView*)view
                                atIndex:(NSUInteger)index
{
   if( view == targetView ) {
      return NO;
   } else if( view == depotView ) {
      return NO;
   }
   NSUInteger stackIndex;
   stackIndex = [self indexOfView:view];
   if( stackIndex == NSNotFound ) {
      return NO;
   }
   
   // Haben wir kein Delegate, so erlauben wir das Ziehen
   if( [self delegate] == nil ) {
      return YES;
   }
   
   // Haben wir eines, so befragen wir es
   SEL selector = @selector(         tableBaizeView:
                            shouldDragCardsAtIndex:
                            fromStackWithIndex: );
   id myDelegate = self.delegate;
   if( [myDelegate respondsToSelector:selector] ) {
      return [delegate         tableBaizeView:self
                       shouldDragCardsAtIndex:index
                           fromStackWithIndex:stackIndex];
   }
   
   return YES;
}
#pragma mark -

#pragma mark Creation
- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
       
       // Zehn obere Stapel
       CardStackView* view;
       playerViews = [[NSMutableArray alloc] init];
       NSUInteger index;
       for( index = 0; index < 10; index++ ) {
          view = [[[CardStackView alloc] 
                   initWithFrame:NSZeroRect] 
                  autorelease];
          [view setAlignment:NSMaxYEdge];
          [self addSubview:view];
          [playerViews addObject:view];
       }
       
       // Depot
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMinXEdge];
       [self addSubview:view];
       depotView = view;
       
       // Ablage
       view = [[[CardStackView alloc] 
                initWithFrame:NSZeroRect] 
               autorelease];
       [view setAlignment:NSMaxXEdge];
       [self addSubview:view];
       targetView = view;
       
       [self resizeSubviews];
    }
    return self;
}

- (void) dealloc
{
   [playerViews release];
   playerViews = nil;
   depotView = nil;
   targetView = nil;
   
   [super dealloc];
}
@end
