#import "ColorControlsController.h"
#import <QuartzCore/QuartzCore.h>

@interface ColorControlsController()

#pragma mark Properties
@property (retain) CIFilter *filter;
@property (readonly) NSImage *originalImage;

- (void)updateOutputImageView;

@end

@implementation ColorControlsController

#pragma mark Properties
@synthesize filter, outputImageView;

@dynamic originalImage;
- (NSImage *)originalImage
{
    return [NSImage imageNamed:@"me"];
}

#pragma mark Creating
- (id)init
{
    self = [super init];
    if(self)
    {
        CIFilter *colorControlsFilter = [CIFilter filterWithName:
                                         @"CIColorControls"];
        [self setFilter:colorControlsFilter];
        [[self filter] setDefaults];
        NSData *data = [[self originalImage] TIFFRepresentation];
        CIImage *image = [CIImage imageWithData:data];
        [[self filter] setValue:image forKey:kCIInputImageKey];
        
    }
    return self;
}

- (void)awakeFromNib
{
    [[self outputImageView] setImage:[self originalImage]];
}

#pragma mark Actions
- (IBAction)takeSaturationFromSender:(id)sender
{
    float saturation = [sender floatValue];
    NSNumber *newValue = [NSNumber numberWithFloat:saturation];
    [[self filter] setValue:newValue
                      forKey:kCIInputSaturationKey];
    [self updateOutputImageView];
}

- (IBAction)takeBrightnessFromSender:(id)sender
{
    float brightness = [sender floatValue];
    NSNumber *newValue = [NSNumber numberWithFloat:brightness];
    [[self filter] setValue:newValue 
                     forKey:kCIInputBrightnessKey];
    [self updateOutputImageView];
}

- (IBAction)takeContrastFromSender:(id)sender
{
    float contrast = [sender floatValue];
    NSNumber *newValue = [NSNumber numberWithFloat:contrast];
    [[self filter] setValue:newValue forKey:kCIInputContrastKey];
    [self updateOutputImageView];
}

- (void)updateOutputImageView
{
    CIImage *outputImage = [[self filter] valueForKey:
                            kCIOutputImageKey];
    
    CGFloat width = CGRectGetWidth([outputImage extent]);
    CGFloat height = CGRectGetHeight([outputImage extent]);
    NSSize size = NSMakeSize(width, height);
    
    NSImage *newImage = [[NSImage alloc] initWithSize:size];
    
    id rep = [NSCIImageRep imageRepWithCIImage:outputImage];
    [newImage addRepresentation:rep];
    [[self outputImageView] setImage:newImage];
    [newImage release];
}

#pragma mark Deallocation
- (void)dealloc
{
    [self setFilter:nil];
    [self setOutputImageView:nil];
    [super dealloc];
}

@end
