//
//  TextAndAdditionalCell.m
//  CustomOutlineView
//
//  Created by Amin Negm-Awad on Fr.,09.07.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "TextAndAdditionalCell.h"

static const CGFloat spacingRatio = 0.3;

@implementation TextAndAdditionalCell
@synthesize additionalCell;

   
- (void)      getMainRect:(NSRect*)mainRect 
        getAdditionalRect:(NSRect*)additionalRect
                forBounds:(NSRect)rect
{
   rect = [super titleRectForBounds:rect];
   if( self.additionalCell ) {
      NSSize additionalSize = [self.additionalCell cellSize];
      CGFloat shrink = additionalSize.width;
      if( shrink < rect.size.width ) {
         NSDivideRect( rect, 
                       additionalRect,
                       mainRect, 
                       shrink, 
                       NSMaxXEdge );
         NSRect spacingRect;
         shrink = additionalSize.height * spacingRatio;
         NSDivideRect( *mainRect, &spacingRect, mainRect, shrink, NSMaxXEdge );
      }
   } else {
      *mainRect = rect;
      *additionalRect = NSZeroRect;
   }
}



- (void)drawInteriorWithFrame:(NSRect)rect
                       inView:(NSView*)control
{
   NSRect mainRect;
   NSRect additionalRect;
   [self	      getMainRect:&mainRect
         getAdditionalRect:&additionalRect 
                 forBounds:rect];
   if( additionalRect.size.width > 0.0 ) { 
      [additionalCell drawInteriorWithFrame:additionalRect
                                     inView:control];
   }
   if( mainRect.size.width > 0.0 ) { 
      [super drawInteriorWithFrame:mainRect inView:control];
   }
}

- (id)copyWithZone:(NSZone*)zone 
{
   TextAndAdditionalCell* copy = [super copyWithZone:zone];
   copy->additionalCell = nil;
   [copy setAdditionalCell:self.additionalCell];
   return copy;
}

- (void) dealloc {
   self.additionalCell = nil;
   [super dealloc];
}
@end
