//
//  BadgeCell.m
//  CustomOutlineView
//
//  Created by Amin Negm-Awad on Fr.,09.07.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "BadgeCell.h"


@implementation BadgeCell
- (NSSize)cellSize {
   NSSize size = [super cellSize];
   size.width += size.height;
   return size;
}

- (NSColor*)badgeColorWithFrame:(NSRect)rect inView:(NSView*)control
{
   NSColor* color;

   if( ![control isKindOfClass:[NSTableView class]] ) { 
      return [super highlightColorWithFrame:rect inView:control];
   }
   
   NSTableView* tableView = (NSTableView*)control; 
   
   // "Normales" Tableview 
   if(    [tableView selectionHighlightStyle] 
       != NSTableViewSelectionHighlightStyleSourceList ) {
      if( self.isHighlighted ) { 
         return [super highlightColorWithFrame:rect 
                                        inView:control];
      } else {
         color = [NSColor controlTextColor];
      }
      
   // Sourceview
   } else {
      // unselektiertes Element 
      if( !self.isHighlighted ) {
         color = [NSColor colorWithCalibratedRed:0.61569
                                           green:0.67059 
                                            blue:0.79608
                                           alpha:1.0];
      // selektiert, aber Fenster inaktiv 
      } else {
         NSWindow* window = control.window; 
         if( !window ) {
            return [super highlightColorWithFrame:rect 
                                           inView:control];
         }
         
         // Fenster ist inaktiv 
         if( !window.isMainWindow ) {
            color = [NSColor colorWithCalibratedRed:0.67451
                                              green:0.67451
                                               blue:0.67451
                                              alpha:1.0];
            
         // (Fenster ist aktiv.) control hat keinen Fokus
         } else if( window.firstResponder != control ) {
            color = [NSColor colorWithCalibratedRed:0.61569
                                              green:0.67059 
                                               blue:0.79608
                                              alpha:1.0];
         // (control hat den Fokus) 
         } else {
            color = [NSColor colorWithCalibratedRed:0.31373 
                                              green:0.53333 
                                               blue:0.80392
                                              alpha:1.0];
         }
      }
   }
   return color;
}
- (void)drawInteriorWithFrame:(NSRect)rect 
                       inView:(NSView*)control
{
   // Kein Wert oder 0-Wert 
   NSString* count = self.stringValue; 
   if( count && [count isEqualToString:@"0"] ) {
      self.stringValue = @"";
      count = nil;
   }
   if( !count || [count isEqualToString:@""] ) {
      [super drawInteriorWithFrame:rect inView:control]; 
      return;
   }
   
   // Die Groesse auf die minimale Breite setzen und platzieren
   NSSize size = [self cellSize];
   
   switch( self.alignment ) { 
      case NSRightTextAlignment:
         rect.origin.x += (NSWidth( rect ) - size.width);
         break;
      case NSCenterTextAlignment: 
         rect.origin.x += (NSWidth( rect ) - size.width)/2.0;
         break;
      case NSLeftTextAlignment: 
      default:
         break;
   }
   rect.origin.y += (NSHeight( rect ) - size.height)/2.0;
   rect.size = size;
   NSRect textRect = NSInsetRect( rect, size.height/2.0, 0.0 );
   
   // Dunkler Hintergrund
   NSColor* darkColor = [self badgeColorWithFrame:rect
                                           inView:control];
   
   // Unselektiert 
   if( self.isHighlighted == NO ) {
      [darkColor setFill];
      [self setTextColor:[NSColor whiteColor]];
      
   // Selektiert: Farben drehen 
   } else {
      [[NSColor whiteColor] setFill];
      [self setTextColor:darkColor];
   }
   
   NSBezierPath* path 
   = [NSBezierPath bezierPathWithRoundedRect:rect
                                     xRadius:size.height/2.0 
                                     yRadius:size.height/2.0];
   [path fill];
   
   [self setHighlighted:NO];
   [super drawInteriorWithFrame:textRect inView:control];
}
@end
