//
//  ClockView.m
//  DockClock
//
//  Created by Amin Negm-Awad on Mo.,09.08.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "ClockView.h"

const CGFloat border = -12;
const CGFloat radius = 6;
const CGFloat thickness = 8;

@implementation ClockView
@synthesize date;
@synthesize attribs;

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if( self ) {
       NSFont* font = [NSFont userFixedPitchFontOfSize:18.0];
       NSColor* color = [NSColor greenColor];
       NSShadow* shadow = [[[NSShadow alloc] init] autorelease];
       [shadow setShadowColor:color];
       [shadow setShadowOffset:NSZeroSize];
       [shadow setShadowBlurRadius:8.0];
       color = [color blendedColorWithFraction:0.4
                                       ofColor:[NSColor whiteColor]];    
    
    
       NSMutableParagraphStyle* paragraphStyle 
       = [[[NSParagraphStyle defaultParagraphStyle]
                             mutableCopy]
                             autorelease]; 
       [paragraphStyle setAlignment:NSCenterTextAlignment];
       self.attribs = [NSDictionary dictionaryWithObjectsAndKeys:
          font,           NSFontAttributeName, 
          color,          NSForegroundColorAttributeName,
          shadow,         NSShadowAttributeName,
          paragraphStyle, NSParagraphStyleAttributeName,
          nil];

       self.date = [NSDate date];
    }
    return self;
}

- (void)drawRect:(NSRect)dirtyRect 
{
   NSDateFormatter *dateFormatter
   = [[[NSDateFormatter alloc] init] autorelease];
   [dateFormatter setDateStyle:NSDateFormatterNoStyle];
   [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
   NSString* text = [dateFormatter stringFromDate:self.date];
   
   NSRect textRect = self.bounds;
   CGSize size = [text sizeWithAttributes:self.attribs];
   textRect.origin.x = NSMidX( textRect ) - size.width / 2.0;
   textRect.origin.y = NSMidY( textRect ) - size.height / 2.0;
   textRect.size = size;
   NSRect borderRect = NSInsetRect( textRect, border, border );
   
   NSBezierPath* border;
   NSGradient* gradient 
   = [[[NSGradient alloc]
                   initWithStartingColor:[NSColor whiteColor] 
                             endingColor:[NSColor darkGrayColor]]
                   autorelease];
   
   border = [NSBezierPath bezierPathWithRoundedRect:borderRect 
                                            xRadius:radius
                                            yRadius:radius];
   
   borderRect = NSInsetRect( borderRect, thickness, thickness );
   NSBezierPath* background 
   = [NSBezierPath bezierPathWithRect:borderRect];
   [border appendBezierPath:background];
   [border setWindingRule:NSEvenOddWindingRule];
   [gradient     	 drawInBezierPath:border
             relativeCenterPosition:NSMakePoint( -0.6, 1.0 )];
   
   
   NSColor* color = [NSColor blackColor];
   color = [color colorWithAlphaComponent:0.5];
   [color setFill];
   [background fill];
   
   [text drawWithRect:textRect 
              options:NSStringDrawingUsesLineFragmentOrigin
           attributes:self.attribs];
}
@end
