//
//  CircleTextContainer.m
//  LayoutStack
//
//  Created by Amin Negm-Awad on Do.,08.07.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "CircleTextContainer.h"

#import "CircleTextView.h"

@implementation CircleTextContainer
- (NSRect)lineFragmentRectForProposedRect:(NSRect)proposedRect 
          sweepDirection:(NSLineSweepDirection)sweep 
          movementDirection:(NSLineMovementDirection)movement 
          remainingRect:(NSRect*)remainingRect
{
   proposedRect 
   = [super lineFragmentRectForProposedRect:proposedRect
                             sweepDirection:sweep 
                          movementDirection:movement
                              remainingRect:remainingRect];
   NSTextView* view = self.textView;
   if( ![view isKindOfClass:[CircleTextView class]] ) {
      return proposedRect;
   }
   NSRect circleRect = [((CircleTextView*)view) circleRect];
   
   // Oberhalb des Kreises: Nichts zu tun 
   if( NSMaxY( proposedRect ) < NSMinY( circleRect ) ) {
      *remainingRect = NSZeroRect; 
      return proposedRect;
   }
   
   if( NSMinY( proposedRect ) > NSMaxY( circleRect ) ) {
      *remainingRect = NSZeroRect;
      return proposedRect;
   }
   
   // Hier: Rechteck liegt in Höhe des Kreises 
   CGFloat width;
   
   // Es beginnt links davon:Dann müssen wir es beschneiden.
   if( NSMinX( proposedRect ) < NSMinX( circleRect ) ) {
      // Das remaining liegt dann rechts vom Kreis, ... 
      // Phasenwinkel an den Schnittpunkten: 
      NSPoint circleCenter 
      = NSMakePoint( NSMidX( circleRect ), NSMidY( circleRect )); 
      CGFloat radius = NSWidth( circleRect ) / 2;
      width = NSMaxX( proposedRect ) - NSMaxX( circleRect );   
      CGFloat upperAngle 
      = acos((NSMinY(proposedRect) - circleCenter.y) / radius); 
      CGFloat lowerAngle = acos((NSMaxY(proposedRect) - circleCenter.y) / radius);
      
      BOOL upperAbove = YES; 
      if( !isnan( upperAngle ) ) {
         upperAbove = upperAngle > M_PI_2;
      }
      
      BOOL lowerAbove = NO; 
      if( !isnan( lowerAngle ) ) {
         lowerAbove = lowerAngle > M_PI_2;
      }
      
      // Auswahl des Schnittpunktes: 
      CGFloat distance; 
      if( upperAbove && lowerAbove ) {
         distance = sin( lowerAngle ) * radius;
      } else if( !upperAbove || !lowerAbove ) { 
         distance = sin( upperAngle ) * radius;
         
      } else { 
         distance = radius;
      }
      
      CGFloat width 
      = NSMaxX( proposedRect ) - circleCenter.x + distance; 
      proposedRect.size.width 
      = circleCenter.x - distance - NSMinX( proposedRect );
      
      // ... wenn ueberhaupt 
      if( width <= 0.0 ) {
         *remainingRect = NSZeroRect; 
      } else {
         *remainingRect = proposedRect; 
         remainingRect->origin.x	= (circleCenter.x + distance); 
         remainingRect->size.width = width;
      }
   
      // Das proposed bis zum Kreis begrenzen. 
      proposedRect.size.width 
      = NSMinX( circleRect) - NSMinX( proposedRect );
   
   } else { 
      *remainingRect = NSZeroRect;
      width = NSMaxX( proposedRect ) - NSMaxX( circleRect ); 
      if( width < 0.0 ) {
         return NSZeroRect; 
/*
      } else {
         proposedRect.origin.x = NSMaxX( circleRect );
         proposedRect.size.width = width;
 */
      }
   }
   return proposedRect;
}

- (BOOL)isSimpleRectangularTextContainer { 
   return NO;
}
@end
