//
//  ScrewCell.m
//  Stackspector
//
//  Created by Amin Negm-Awad on Fr.,09.07.10.
//  Copyright 2010 Dr. Pribilla Kaldenhoff Negm. All rights reserved.
//

#import "ScrewCell.h"
#import "ScrewDisclosureCellSupport.h"

@implementation ScrewCell
-(void)setState:(NSInteger)value {
   if( (value != self.state ) && (self.controlView != nil) ) { 
      NSView* control = self.controlView;
      if( [control isKindOfClass:[ScrewDisclosure class]] ) {
         ScrewDisclosure* disclosure = (ScrewDisclosure*)control;
         if( value == NSOnState ) {
            [disclosure showContent]; 
         } else {
            [disclosure hideContent];
         }
      }
   }
   [super setState:value];
}

- (BOOL)trackMouse:(NSEvent*)event 
            inRect:(NSRect)frame
            ofView:(NSView*)control 
      untilMouseUp:(BOOL)mouseUp
{
   NSPoint location = [event locationInWindow];
   location = [control convertPoint:location fromView:nil];
   NSRect barRect;
   NSRect contentRect;
   NSDivideRect(  frame,
                 &barRect,
                 &contentRect,
                  [self cellSize].height,
                  NSMaxYEdge );
   if( NSPointInRect( location, barRect ) ) { 
      return [super trackMouse:event
                        inRect:barRect
                        ofView:control
                  untilMouseUp:mouseUp];
   } else { 
      return NO;
   }
}

- (BOOL)startTrackingAt:(NSPoint)location 
                 inView:(NSView*)control
{
   NSLog( @"startTrackingAt: %@", NSStringFromPoint( location ) );
   return YES;
}

- (BOOL)continueTracking:(NSPoint)lastLocation 
                      at:(NSPoint)location 
                  inView:(NSView *)control
{
   NSLog( @"continueTracking:at: %@", 
          NSStringFromPoint ( location ) );
   return YES;
}

- (void)stopTracking:(NSPoint)lastLocation 
                  at:(NSPoint)location 
              inView:(NSView *)control
           mouseIsUp:(BOOL)flag
{
   NSLog( @"%d", flag );
}

- (void)drawInteriorWithFrame:(NSRect)frame 
                       inView:(NSView*)control
{
   NSRect barRect;
   NSRect contentRect;
   NSDivideRect(  frame,
                 &barRect,
                 &contentRect,
                  [self cellSize].height,
                  NSMaxYEdge );
   barRect = NSInsetRect( barRect, 1.0, 1.0 );
   
   // Zeichne Hintergrund 
   NSColor* backgroundColor;
   if( self.backgroundStyle == NSBackgroundStyleDark ) {
      backgroundColor = [NSColor darkGrayColor];
   } else {
      backgroundColor = [NSColor lightGrayColor];
   }
   [backgroundColor setFill]; 
   
   NSRectFill( barRect );
   
   // Elemente 
   NSRect screwRect; 
   NSRect titleRect; 
   NSDivideRect(  barRect,
                 &screwRect,
                 &titleRect,
                  barRect.size.height,
                  NSMinXEdge );
   
   // Schraube zeichnen 
   //	Kreis 
   [[NSGraphicsContext currentContext] saveGraphicsState];
   if( [self state] == NSOnState ) {
      NSShadow* shadow = [[[NSShadow alloc] init] autorelease];
      NSSize offset;
      offset.width = +NSWidth( screwRect ) / 10.0;
      offset.height = -NSHeight( screwRect ) / 10.0;
      [shadow setShadowOffset:offset];
      [shadow setShadowBlurRadius:4.0];
      [shadow setShadowColor:[NSColor blackColor]];
      [shadow set];
      screwRect = NSInsetRect( screwRect, 3.0, 3.0 ); 
   } else {
      screwRect = NSInsetRect( screwRect, 4.0, 4.0 );
   }
   NSBezierPath* circle;
   circle = [NSBezierPath bezierPathWithOvalInRect:screwRect];
   
   [[NSColor controlColor] setFill];
   [circle fill];
   [[NSGraphicsContext currentContext] restoreGraphicsState];
   
   //	Schlitz
   NSBezierPath* line;
   line = [[[NSBezierPath alloc] init] autorelease];
   NSPoint point;
   point.x = NSMidX( screwRect );
   point.y = NSMinY( screwRect );
   [line moveToPoint:point];
   point.y = NSMaxY( screwRect );
   [line lineToPoint:point];
   
   if( self.state == NSOnState ) {
      NSAffineTransform* transform 
      = [NSAffineTransform transform];
      [transform translateXBy:+NSMidX( screwRect )
                          yBy:+NSMidY( screwRect )];
      [transform rotateByDegrees:90.0];
      [transform translateXBy:-NSMidX( screwRect )
                       yBy:-NSMidY( screwRect )];
      [line transformUsingAffineTransform:transform];
   }
   
   [[NSColor blackColor] setStroke];
   [line stroke];
   
   // Titel ausgeben
   [super drawInteriorWithFrame:titleRect inView:control];
}

- (void)drawWithFrame:(NSRect)frame inView:(NSView*)control 
{
   if( [self isBordered] ) { 
      NSRect barRect;
      NSRect contentRect;
      NSDivideRect(  frame,
                    &barRect, 
                    &contentRect, 
                     [self cellSize].height, 
                     NSMaxYEdge );
      [[NSColor blackColor] set];
      NSFrameRect( barRect );
      NSFrameRect( frame );
   }
   [self drawInteriorWithFrame:frame inView:control];
}

- (id)initTextCell:(NSString*)title {
   self = [super initTextCell:title]; 
   if( self ) {
      [self setBordered:YES]; 
      [self setBackgroundStyle:NSBackgroundStyleLight];
   } 
   return self;
}
@end
